#include <stdio.h>
#include <math.h>
#include "geom.h"
/*
    Functions in this code were incorporated from Adrian Baddeley:
	$Revision: 2.1 $ 	$Date: 90/12/23 21:55:35 $

 * Revisions here done by Dan Lofgreen, CIC, 2006
 *
# /////////////////////////////////////////////
# AUTHOR: Adrian Baddeley, CWI, Amsterdam, 1991.
# 
# This software is distributed free
# under the conditions that
# 	(1) it shall not be incorporated
# 	in software that is subsequently sold
# 	(2) the authorship of the software shall
# 	be acknowledged in any publication that 
# 	uses results generated by the software
# 	(3) this notice shall remain in place
# 	in each file.
# //////////////////////////////////////////////

*/

static double pi = 3.141592653589793;

/* Factor of 4 * pi * r * r IS ALREADY TAKEN OUT */

void sphesfrac(points,box0,box1,rbins,numr,numpts,fs)
int numr, numpts;
double *points, *box0, *box1, *rbins, *fs;
{
    double *x, *y, *z;
    double sum, r, p[3], q[3];
    double a1(), a2(), a3();
    int a, i, j, ri, count=0;
    
    // Get the pointers to the arrays corresponding to x,y and z
    x = points;
    y = &points[numpts];
    z = &points[2*numpts];
    
    // Go through each point and calculate the correction factor
    for(a = 0; a<numpts; a++)
    {
        // Calculate the correction factor for each radius
        for(ri = 0; ri<numr; ri++)
        {
            p[0] = x[a] - box0[0];
            p[1] = y[a] - box0[1];
            p[2] = z[a] - box0[2];
            
            q[0] = box1[0] - x[a];
            q[1] = box1[1] - y[a];
            q[2] = box1[2] - z[a];
            
            
            sum = 0.0;
            r = rbins[ri];

            for(i = 0; i < 3; i++)
            {
                sum += a1(p[i],r) + a1(q[i],r);
            }
            
            for(i = 0; i < 2; i++)
                for(j = i+1; j <= 2; j++)
            {
                sum -= a2(p[i], p[j], r) + a2(p[i], q[j], r)
                + a2(q[i], p[j], r) + a2(q[i], q[j], r);
                }

            sum += a3(p[0], p[1], p[2], r) + a3(p[0], p[1], q[2], r);
            sum += a3(p[0], q[1], p[2], r) + a3(p[0], q[1], q[2], r);
            sum += a3(q[0], p[1], p[2], r) + a3(q[0], p[1], q[2], r);
            sum += a3(q[0], q[1], p[2], r) + a3(q[0], q[1], q[2], r);
            
            fs[count] = 1.0 - sum;
            count++;
        }
    }
}

double 
a1(t, r)
double t, r;
{
	/* This is the function A1 divided by 4 pi r^2 */

	if(t >= r) 
		return(0.0);

	return((1 - t/r) * 0.5);
}

double
a2(t1, t2, r)
double t1, t2, r;
{
	double c2();
	/* This is A2 divided by 4 pi r^2 because c2 is C divided by pi */
	
	return(c2( t1 / r, t2 / r) / 2.0);
}

double
a3(t1, t2, t3, r)
double t1, t2, t3, r;
{
	double c3();
	/* This is A3 divided by 4 pi r^2 because c3 is C divided by pi */

	return(c3(t1 / r, t2 / r, t3 / r) / 4.0);
}

double 
c2(a, b)
double a, b;
{
	double z, z2;
	double sqrt(), atan2(), c2();

	/* This is the function C(a, b, 0) divided by pi 
		- assumes a, b > 0  */

	if( ( z2 = 1.0 - a * a - b * b) < 0.0 )
		return(0.0);
	z = sqrt(z2);
	return((atan2(z, a * b) - a * atan2(z, b) - b * atan2(z, a)) / pi);
}

double
c3(a, b, c)
double a, b, c;
{
	double sqrt(), atan2();
	double za, zb, zc, sum;
	/* This is C(a,b,c) divided by pi. Arguments assumed > 0 */

	if(a * a + b * b + c * c >= 1.0)
		return(0.0);

	za = sqrt(1 - b * b - c * c);
	zb = sqrt(1 - a * a - c * c);
	zc = sqrt(1 - a * a - b * b);

	sum =  atan2(zb, a * c) + atan2(za, b * c) 
		+ atan2(zc, a * b)
		- a * atan2(zb, c)
		+ a * atan2(b, zc)
		- b * atan2(za, c)
		+ b * atan2(a, zc)
		- c * atan2(zb, a)
		+ c * atan2(b, za);

	return(sum / pi  - 1);
}
