function varargout = SpatialAnalysis(varargin)
% SPATIALANALYSIS 3D Spatial Analysis software
%  SpatialAnalysis will start the Spatial Analysis 3D software.  The
%  softare must first be installed to use it.
%
%  SpatialAnalysis -setup  Will set up the SpatialAnalysis software.  The
%  setup feature will initialize some application preferences and compile
%  the C routines into Matlab mex functions
%
% See also: INSTALLSPATIALANALYSIS, ABOUTSPATIALANALYSIS, NEARESTNEIGHBOR3, 
% AUTOCORRELATION3, VORONOIANAL3, F3D, G3D, K3D

%  Below is the help information generated by the guide editor in Matlab
%
% SPATIALANALYSIS Control Code for the main Spatial Analysis Window
%      SPATIALANALYSIS, by itself, creates a new SPATIALANALYSIS or raises the existing
%      singleton*.
%
%      H = SPATIALANALYSIS returns the handle to a new SPATIALANALYSIS or the handle to
%      the existing singleton*.
%
%      SPATIALANALYSIS('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in SPATIALANALYSIS.M with the given input arguments.
%
%      SPATIALANALYSIS('Property','Value',...) creates a new SPATIALANALYSIS or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before SpatialAnalysis_OpeningFunction gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to SpatialAnalysis_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help SpatialAnalysis

% Last Modified by GUIDE v2.5 03-May-2007 07:15:19

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @SpatialAnalysis_OpeningFcn, ...
    'gui_OutputFcn',  @SpatialAnalysis_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin & isstr(varargin{1})
    switch(varargin{1})
        case '-setup'
            SUCCESS = SetupSpatialAnalysis();
            return
        case '-root'
            % This case will log the user in as the super-user or the root
            % user with some extra privledges.
            setpref('SpatialAnalysis','User','root')
        otherwise
            gui_State.gui_Callback = str2func(varargin{1});
    end
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

% --- Executes just before SpatialAnalysis is made visible.
function SpatialAnalysis_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to SpatialAnalysis (see VARARGIN)

% Check to see if we are deleting the figure
if(strcmp(get(hObject,'BeingDeleted'),'on'))
    return
end

% Check to see if the user has logged in as the root user
user = getpref('SpatialAnalysis','User');
switch(user)
    case 'root'
        set(handles.DelaunayMovieMenu,'enable','on')
    otherwise
        set(handles.DelaunayMovieMenu,'enable','off')
end

% Check to see if the recently opened file preferences have been set
for i = 1:4
    if(~ispref('RecentFiles',['File',num2str(i)]))
        addpref('RecentFiles',['File',num2str(i)],'')
    end
end

% Choose default command line output for SpatialAnalysis
handles.output = hObject;

% Initialize the wait bar
hP = InitializeSpatialAnalysisWaitBar(hObject, eventdata, handles, varargin);
handles.WaitBar = hP;

% Call the update recent files function
UpdateRecentFiles(hObject, eventdata, handles)

% Initialize the Data Table
InitializeDataTable(hObject, eventdata, handles)

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes SpatialAnalysis wait for user response (see UIRESUME)
% uiwait(handles.SpatialAnalysisFig);

% Define the default options structure
options.MinX = 0;
options.MaxX = 1000;
options.MinY = 0;
options.MaxY = 1000;
options.MinZ = 0;
options.MaxZ = 1000;
options.MarkerSize = 13;
options.Precision = 4;
options.Transparency = .5;
if(ispc)
    options.Units = 'm';
else
    options.Units = 'um';
end
options.AspectRatio = 'Auto';

options.NumNNBins = 15;
options.NNBinSize = 10;
options.NumDelaunaySegLenBins = 15;
options.DelaunaySegLenBinSize = 20;
options.NumDelaunayAreaBins = 15;
options.DelaunayAreaBinSize = 1e3;
options.NumDelaunayVolBins = 15;
options.DelaunayVolBinSize = 2.5e4;
options.NNFilter = 'Infected Points';

options.NumVoronoiVolBins = 15;
options.VoronoiVolBinSize = 2e5;
options.VoronoiVolMaxFreq = 30;
options.NumVoronoiAreaBins = 15;
options.VoronoiAreaBinSize = 1e4;
options.VoronoiAreaMaxFreq = 30;
options.VoronoiFilter = 'Infected Points';

options.NumACBins = 20;
options.MaxRadiusAC = 200;
options.MarkerSizeAC = 10;
options.EdgeCorrectionAC = 'Fractional';
options.FilterAC = 'None';
options.PLOTAC = 1;
options.ACPhiBins = 18;
options.ACThetaBins = 36;

options.fdivisions = 100;
options.FFcnRange = 200;
options.FFcnNumPoints = 30;
options.GFcnRange = 200;
options.GFcnNumPoints = 30;
options.KFcnRange = 200;
options.KFcnNumPoints = 30;

options.NumRandPoints = 1000;
options.RandomAvgDist = 22;
options.RandomMinDist = 2;
options.RandomSTD = 2;
options.SimulatedDataType = 'Random';

% Define a PlotData structure
PlotData.handles = [];
PlotData.type = 'Points';
PlotData.style = '';

data{1} = [];
data{2} = options;
data{3} = PlotData;

% Save the data and set the renderer mode to auto since we're having so
% much trouble with the renderer
set(gcf,'UserData',data,'Renderer','opengl')

% Create a new set of random data
ERROR = NewRandomSet(hObject, eventdata, handles);

% Check to see if we should display the Tip of the day
if(ispref('SpatialAnalysisTips','Display'))
    if(getpref('SpatialAnalysisTips','Display'))
        % If we should display the tips, then call the function
        TipOfTheDay
    end
else
    % If we don't know if we should display it or not, then do
    TipOfTheDay
end

% -------------------------------------------------------------------------
function varargout = SpatialAnalysis_OutputFcn(hObject, eventdata, handles)
% Get default command line output from handles structure


% --------------------------------------------------------------------
function OpenMenu_Callback(hObject, eventdata, handles, varargin)
% Open Menu Function
data = get(gcf,'UserData');
results = data{1};

% See if a flag was passed that indicates we opened a recently opened file
if(length(varargin))
    % The callback object (recent file menu item) has the filename
    filename = get(gcbo,'UserData');
    [pathname,filename,ext,versn] = fileparts(filename);
    filename = [filename,ext];
    pathname = [pathname,filesep];
    switch(ext)
        case '.xls'
            filterindex = 1;
        case '.csv'
            filterindex = 2;
        case '.txt'
            filterindex = 3;
        case '.sa3'
            filterindex = 4;
        otherwise 
            filterindex = 0;
    end
else
    % If a filename was not passed, then open the get file dialog box
    curdir = pwd;
    cd(results.DataPath)
    [filename,pathname,filterindex] = uigetfile({...
        '*.xls','Excel Worksheet';...
        '*.csv','Comma Separated Values';...
        '*.txt','Text files';...
        '*.sa3','SpatialAnalysis3D files'},...
        'Open Data File...',...
        '*.xls');
    cd(curdir)

    % IF THE OPEN MENU WAS CANCELED, A ZERO IS RETURNED
    if(filterindex == 0)
        return
    end
end

% Echo loading the file
SpatialAnalysisMessage(['Loading ',pathname,filename])

% Load the file depending on the type
try
    switch(filterindex)
        case 1
            % This is for *.xls files
            [numeric,text,raw] = xlsread([pathname,filename]);
            % Clear the results structure
            results = ClearResults(hObject,eventdata,handles,results,length(numeric));
            for i = 1:length(text)
                header = text{i};
                header = header(setdiff(1:length(header),strfind(header,' ')));
                results = setfield(results,header,numeric(:,i));
            end
            results.points = [results.X,results.Y,results.Z];
        case 2
            % This is for *.csv files            
            results = ClearResults(hObject,eventdata,handles,results,1);
            [results.ObjectNumber,results.ObjectType,results.Time,results.X,...
                results.Y,results.Z,results.Area,results.Compactness,...
                results.Circularity,results.Eccentricity,results.Solidity,...
                results.SurfaceArea,results.Volume,results.Sphericity,...
                results.Other] = textread([pathname,filename],...
                '%d%s%f%f%f%f%f%f%f%f%f%f%f%f%f','delimiter',',','headerlines',2);
            results.points = [results.X,results.Y,results.Z];
        case 3
            % This is for *.txt files
            % Get the headers
            fid = fopen([pathname,filename]);
            text = fgetl(fid);
            fclose(fid);
            % Read in the tab-delimited text
            numeric = dlmread([pathname,filename],'\t',1,0);
            % Find the position of each tab.  Use a zero at the begining so
            % that we can pull out the first header with our same algorithm
            A = [0,findstr(text,char(9))];

            % Clear the results structure
            results = ClearResults(hObject,eventdata,handles,results,length(numeric));
  
            % Go through each tab and pull out the header name
            for i = 2:length(A)
                header = text(A(i-1)+1:A(i)-1);
                % Take out any spaces in the header
                header = header(setdiff(1:length(header),strfind(header,' ')));
                results = setfield(results,header,numeric(:,i-1));
            end

            % Save the point coordinates as a matrix
            results.points = [results.X,results.Y,results.Z];
    
        case 4
            % This is for *.sa3 files
            load([pathname,filename],'-mat')
            results = data{1};
            
        otherwise
            SpatialAnalysisMessage(['ERROR: Improper file type ',pathname, filename])
            return
    end
catch
    SpatialAnalysisMessage(['ERROR: Could not load ',pathname,filename])
    SpatialAnalysisMessage(['     ',lasterr])
    return
end

% Check to see if any points were loaded
if(isempty(results.points) | isempty(results.X) | isempty(results.Y) | ...
        isempty(results.Z))
    SpatialAnalysisMessage(['ERROR: ',pathname,filename,...
        ' contains improper X,Y, or Z data'])
    return
end

% Update the Recently Opened File list
UpdateRecentFiles(hObject, eventdata, handles, [pathname,filename])

% Set the name of the window to include the filename
A = findstr(filename,'.');
if(~isempty(A))
    filename = filename(1:A-1);
end
set(gcbf,'Name',['3D Spatial Analysis - ',filename]);

% Save the data path
results.DataPath = pathname;

% Find the indicies that correspond to the convex hull
K = convhulln(results.points);
K = unique(K);

% Save the indices of the convex hull
results.ConvexHullIndices = K;

data{1} = results;
set(gcf,'UserData',data)

% Determine the extent of the data
FindDataExtent(hObject, eventdata, handles)

% After the data has been loaded, plot the data
PlotPoints(hObject, eventdata, handles)

% Update Summary Table
UpdateSummaryTable(hObject, eventdata, handles)

% Enable/disable the false color menu items
EnableFalseColorMenues(hObject, eventdata, handles)

% --------------------------------------------------------------------
function CloseMenu_Callback(hObject, eventdata, handles)
% This function will close the current file that is open
data = get(gcf,'UserData');
results = data{1};

% Call the Clear Results function to clear & initialize the results
results = ClearResults(hObject,eventdata,handles,results,1);
results.points = [NaN NaN NaN];
results.CurPoint = 0;

data{1} = results;
set(gcf,'UserData',data)

% Set the name of the figure back to the default
set(gcbf,'Name','3D Spatial Analysis');

% After the data has been loaded, plot the data
PlotPoints(hObject, eventdata, handles, 0)

% Clear the data table
ClearDataTable(hObject, eventdata, handles)

% Make sure the data table is in view
SelectHistogramDataPanel(hObject, eventdata, handles, 'DataPanel')

% --------------------------------------------------------------------
function SaveMenu_Callback(hObject, eventdata, handles)
% This function will save the data to the filename already chosen
data = get(gcf,'UserData');
results = data{1};

% Check to see if a filename is stored in the results structure
if(isempty(results.filename))
    SaveAsMenu_Callback(hObject, eventdata, handles)
    return
else
    save(results.filename,'-mat','data')
end


% --------------------------------------------------------------------
function SaveAsMenu_Callback(hObject, eventdata, handles)
% This function will open the save as window
data = get(gcf,'UserData');
results = data{1};

[filename, pathname] = uiputfile('*.sa3', 'Save to file...');

% Check to see if the user canceled
if(filename)
    % Get all the parts of the filename
    [pathstr,name,ext,versn] = fileparts(filename);
    
    % Make sure the file extension is the .sa3 extention
    if(~strcmp(ext,'.sa3'))
        filename = [name,'.sa3'];
    end
    
    % Build the filename
    filename = [pathname,filename];
    
    % Update the Name of the Spatial Analysis window
    set(gcbf,'Name',['3D Spatial Analysis - ',name]);
    
    % Save the filename in the results structure
    results.filename = filename;
    
    % Save the data
    data{1} = results;
    set(gcf,'UserData',data)

    % Save the file with this filename
    save(filename,'-mat','data')
end


% --------------------------------------------------------------------
function ExportMenu_Callback(hObject, eventdata, handles)
% Will call the export dialog window
data = get(gcf,'UserData');
results = data{1};
options = data{2};

% Check to see if we have any points to do calculations on
if(length(results.X)==0)
    hED = errordlg('Not enough points to perform any analysis',...
        '3D Spatial Analysis ERROR','modal');
    waitfor(hED)
    return
end

% Check to see if we perfomed the nearest neighbor analysis
if(~results.NNCALC)
    results = NearestNeighborCalculation(hObject, eventdata, handles, results);
    results.NNCALC = 1;
end

% If we have not performed the voronoi calculation yet, do that now
if(~results.VORONOICALC)
    results = VoronoiDiagramsCalculation(hObject, 0, handles, results);
    results.VORONOICALC = 1;
end

% Check to see if we performed the autocorrelation
if(~results.ACCALC)
    options.WaitBar = [];       % Don't update the waitbar
    options.AxesHandle = [];    % Don't plot the data
    options.EdgeCorrection = options.EdgeCorrectionAC;
    options.Filter = options.FilterAC;
    options.MaxRadius = options.MaxRadiusAC;
    options.NumBins = options.NumACBins;
    options.MarkerSize = options.MarkerSizeAC;
    SpatialAnalysisMessage('Performing autocorrelation analysis...')
    [results.ACBins,results.ACCount,results.ACBinVolume,...
        results.SampleVolume,results.ACIndices,ACStats] = ...
        autocorrelation3(results.points,options);
    SpatialAnalysisMessageDONE

    % Save the results statistics if there were any
    if(~isempty(ACStats))
        results.ACDensity = ACStats.density;
        results.ACDc = ACStats.Dc;
        results.ACreff = ACStats.reff;
        results.ACrm = ACStats.rm;
        results.ACk = ACStats.k;
        results.ACp = ACStats.p;
    end
end

% Calculate the NN Hist statistics
results = NNHistogramCalc(hObject, eventdata, handles, results);

% Calculate the Delaunay statistics
results = DelaunayHistogramCalc(hObject, eventdata, handles, results);

% Calculate the Voronoi statistics
results = VoronoiHistogramCalc(hObject, eventdata, handles, results);

% Let the user know you are performing the F,G,K function statistics
SpatialAnalysisMessage('Performing F,G,K analyses...')

% Calculate the F-function statistics
results = FFunctionCalc(hObject, eventdata, handles, results);

% Calculate the G-function statistics
results = GFunctionCalc(hObject, eventdata, handles, results);

% Calculate the K-function statistics
results = KFunctionCalc(hObject, eventdata, handles, results);

% Tell the user yo uare done
SpatialAnalysisMessageDONE

% Save the results
data{1} = results;
set(gcf,'UserData',data);

% Set the names of the data to be saved
names = {'X','Y','Z','Area','Volume','Eccentricity','Circularity',...
    'Sphericity',...
    'Number of Points','Convex Hull Indices','Number of Voronoi Vertices',...
    'Number of Voronoi Facets','Number of Near Neighbors',...
    'Nearest Neighbor Distance','Near Neighbor Max Distance',...
    'Near Neighbor Avg Distance','Near Neighbor Class',...
    'Filtered Nearest Neighbors','Nearest Neighbor Bin Edges',...
    'Nearest Neighbor Bin Count','Nearest Neighbor Bins',...
    'Delaunay Segment Lengths','Delaunay Facet Areas',...
    'Delaunay Tetrahedron Volumes','Delaunay Min Facet Area',...
    'Delaunay Max Facet Area','Delaunay Min Tetrahedron Volume',...
    'Delaunay Max Tetrahedron Volume','Number of Delaunay Facets',...
    'Filtered Delaunay Volumes','Delaunay Volume Bin Edges',...
    'Delaunay Volume Bin Count','Delaunay Volume Bins',...
    'Filtered Delaunay Areas','Delaunay Area Bin Edges',...
    'Delaunay Area Bin Count','Delaunay Area Bins',...
    'Filtered Delaunay Segment Lengths','Delaunay Segment Length Bin Edges',...
    'Delaunay Segmant Length Bin Count','Delaunay Segment Length Bins',...
    'Voronoi Total Area','Voronoi Area Bin Edges','Voronoi Area Bin Count',...
    'Voronoi Total Volume','Voronoi Volume Bin Edges',...
    'Voronoi Volume Bin Count','Voronoi Class','Autocorrelation Bin Edges',...
    'Autocorrelation Bin Count','Sample Volume',...
    'Autocorrelation Bin Volume','Autocorrelation Reliability Factor',...
    'Autocorrelation Packing Factor','Autocorrelation Effective Radius',...
    'Autocorrelation Density','Autocorrelation Critical Density',...
    'Autocorrelation Max Radius',...
    'K-Function Distance','K-Function K(d)','K-Function Poisson',...
    'G-Function Distance','G-Function G(d)','G-Function Poisson',...
    'F-Function Distance','F-Function F(d)','F-Function Poisson'};


% Create a cell array that holds the data to be saved
data2 = {results.X,results.Y,results.Z,results.Area,results.Volume,...
    results.Eccentricity,results.Circularity,results.Sphericity,...
    results.NumPoints,results.ConvexHullIndices,...
    results.NumVoronoiVertices,results.NumVoronoiFacets,results.NumNN,...
    results.MinNNDist,results.MaxNNDist,results.AvgNNDist,results.NNClass,...
    results.NNDistHist,results.NNBinEdges,results.NNBinCount,results.NNBins,...
    results.DelaunaySegLen,results.DelaunayArea,results.DelaunayVolume,...
    results.DelaunayMinArea,results.DelaunayMaxArea,results.DelaunayMinVol,...
    results.DelaunayMaxVol,results.DelaunayNumFacets,...
    results.DelaunayVolume(results.DelaunayVolHistIndex),...
    results.DelaunayVolBinEdges,results.DelaunayVolBinCount,...
    results.DelaunayVolBins,...
    results.DelaunayArea(results.DelaunayAreaHistIndex),...
    results.DelaunayAreaBinEdges,results.DelaunayAreaBinCount,...
    results.DelaunayAreaBins,...
    results.DelaunaySegLen(results.DelaunaySegLenHistIndex),...
    results.DelaunaySegLenBinEdges,results.DelaunaySegLenBinCount,...
    results.DelaunaySegLenBins,...
    results.VoronoiArea,results.VoronoiAreaBinEdges,...
    results.VoronoiAreaBinCount,results.VoronoiVolume,...
    results.VoronoiVolBinEdges,results.VoronoiVolBinCount,...
    results.VoronoiClass,results.ACBins,results.ACCount,...
    results.SampleVolume,results.ACBinVolume,results.ACk,...
    results.ACp,results.ACreff,...
    results.ACDensity,results.ACDc,results.ACrm,...
    results.kr,results.ky,results.kpoisson,...
    results.gr,results.gy,results.gpoisson,...
    results.fr,results.fy,results.fpoisson};

% Call the save selected parameters GUI
saveselectedparams(names,data2)

% --------------------------------------------------------------------
function PrintMenu_Callback(hObject, eventdata, handles)
% Print Window

% Open the print dlg box
printdlg(gcf)


% --------------------------------------------------------------------
function PrintPreviewMenu_Callback(hObject, eventdata, handles)
data = get(gcf,'UserData');
options = data{2};

% Create a list of UI objects that should be made invisible
invislist = [handles.SpatialAnalysisWaitBar;handles.MSGBox;...
    handles.TimePanel;handles.WaitBarPanel;handles.TimeTXT;...
    handles.SpatialAnalysisWaitBar;...
    get(handles.SpatialAnalysisWaitBar,'children')];

% Turn the list visible off 
set(invislist,'visible','off')

% Open the print preview dlg box
hP = printpreview(gcf);

% Wait for the print preview window to be destroyed.  The maximum wait time
% is 3 min
uiwait(hP,180)

% Turn the list visible on 
set(invislist,'visible','on')


% --------------------------------------------------------------------
function ExitMenu_Callback(hObject, eventdata, handles)
% Exit Menu function

% Just call the close requestion function
SpatialAnalysisFig_CloseRequestFcn(hObject, eventdata, handles)

%--------------------------------------------------------------------------
function CheckSendFigureMenu(hObject, eventdata, handles)
% Check to see if the histogram axis is visible.  If it is, then make the
% send axes 2 menu item enabled.

if(strcmp(get(handles.HistogramAxes,'visible'),'on'))
    set(handles.SendAxes2Menu,'enable','on')
else
    set(handles.SendAxes2Menu,'enable','off')
end


% --------------------------------------------------------------------
function varargout = SendFigureMenu_Callback(hObject, eventdata, handles, ax)
% Send the plot axis to a new figure menu.  ax defines which axis to send

% Initialize the colorbar's handle to an empty matrix
hCB1 = [];

% Select the Plot Axes
if(ax==1)
    % If the user asked to send the left axes (1)
    if(strcmp(get(handles.PlotAxes,'visible'),'on'))
        SelectAxes(hObject, eventdata, handles, 'PlotAxes')
    else
        SelectAxes(hObject, eventdata, handles, 'AutocorrelationAxes')
    end

    % Check to see if we have a colorbar
    hCB1 = findobj('tag','Colorbar');

else
    % If the user asked to send the right axes (2)
    axes(handles.HistogramAxes)
end

% Remember the current axis and the associated colormap
curaxes = gca;
cmap = colormap;
xlim = get(gca,'xlim');
ylim = get(gca,'ylim');
zlim = get(gca,'zlim');

% Open a new figure and create an axis at position pos
hNewFig = figure;
pos = [0.1300    0.1100    0.7750    0.8150];

% Copy the "current" axis to this new figure & axis
C = copyobj(curaxes,handles.SpatialAnalysisFig);
set(C,'parent',hNewFig,'units','normalized')
set(C,'position',pos,'xlim',xlim,'ylim',ylim,'zlim',zlim)
colormap(cmap)

% If we had a colorbar in the other plot, then create one here
if(~isempty(hCB1))
    hCB2 = colorbar;
    colorbartitle = get(get(hCB1,'UserData'),'string');
    if(isempty(colorbartitle))
        colorbartitle = get(curaxes,'UserData');
    end
    title(hCB2,colorbartitle);
    set(hCB2,'YTickLabel',get(hCB1,'YTickLabel'))
end
    
% Output the new figure's handle
if(nargout==1)
    varargout{1} = hNewFig;
end


% --------------------------------------------------------------------
function CopyFigureMenu_Callback(hObject, eventdata, handles)
% Call the built in function to copy the figure to the clipboard
editmenufcn('EditCopyFigure')

% -------------------------------------------------------------------------
function SpatialAnalysisFig_CloseRequestFcn(hObject, eventdata, handles)
% This function is called when the user asks to close the figure
delete(gcbf);
setpref('SpatialAnalysis','User','user')

%--------------------------------------------------------------------------
% ANALYSIS MENU

% -------------------------------------------------------------------------
function VoronoiDiagramsMenu_Callback(hObject, eventdata, handles)
% This function will plot the voronoi diagrams for each point
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Check to see if we have any points to do calculations on
if(length(results.X)==0)
    hED = errordlg('Not enough points to perform a Voronoi calculation',...
        '3D Spatial Analysis ERROR','modal');
    waitfor(hED)
    return
end

% Need to check if the points have been plotted, if not, plot them
if(~strcmp(PlotData.type,'Voronoi'))
    PlotPoints(hObject, eventdata, handles,0)

    % No need to change the data table if it has already been changed
    if(get(handles.DataTable2Title,'value')~=4)
        set(handles.DataTable2Title,'value',4)
        DataTable2Title_Callback(hObject, eventdata, handles)
    end
end

% If we have not performed the voronoi calculation yet, do that now
if(~results.VORONOICALC)
    results = VoronoiDiagramsCalculation(hObject, eventdata, handles, results);
end

% Get the points, the current point, C and V from the voronoi calculation
p = results.points;
a = results.CurPoint;
C = results.C;
V = results.V;

% Get the indices of the verticies of the current point
indices = C{a};

% Delete any previous items that need to be deleted
A = find(ishandle(PlotData.handles));
if(any(A))
    delete(PlotData.handles(A))
end

% Select the correct axis
SelectAxes(hObject, eventdata, handles, 'PlotAxes')
FixAxisLimits(hObject, eventdata, handles, 'PlotAxes')
hold on

% If we are on a normal voronoi point OR we are on an infected point, but
% the filter is only set to convex hull, then plot the voronoi diagram
CONTINUE = 0;
switch(results.VoronoiClass(a))
    case 1
        % Normal Point
        CONTINUE = 1;
    case 2
        % Double infected Point
        if(strcmp(options.VoronoiFilter,'Convex Hull') | ...
                strcmp(options.VoronoiFilter,'Infected Points'))
            CONTINUE = 1;
        end
    case 3
        % Infected point
        if(strcmp(options.VoronoiFilter,'Convex Hull'))
            CONTINUE = 1;
        end
    case 4
        % Point on the convex hull
        % We will not continue
    case 5
        % Other
        % We will not continue
end

if(CONTINUE)

    % Get the "hull" data for the given point.  The hull points are the
    % voronoi vertices
    X = V(indices,:);
    K = results.K{a};

    % Create a patch (facet) for each voronoi surface
    for b = 1:length(K)
        h(b) = patch(X(K(b,:),1),X(K(b,:),2),X(K(b,:),3),'g');
    end

    % Set the transparency
    alpha(h,options.Transparency)

    % Plot the point we are currently on
    h(b+1) = plot3(p(a,1),p(a,2),p(a,3),'*r','markersize',options.MarkerSize);
    drawnow
else
    % If there is some issue with the point, simply display the comment and
    % plot the point of interest in red
    if(results.VoronoiClass(a)==5)
        SpatialAnalysisMessage(results.Comment{a})
    end
    h(1) = plot3(p(a,1),p(a,2),p(a,3),'*r','markersize',options.MarkerSize);
end

% Set the title for the plot
title('Voronoi Domain')

% Update the PlotData
PlotData.handles = h;
PlotData.type = 'Voronoi';
PlotData.style = '';

% Save the data
data{1} = results;
data{3} = PlotData;
set(gcf,'UserData',data)

% Update the Data table
UpdateDataTable(hObject, eventdata, handles)

%--------------------------------------------------------------------------
function results = VoronoiDiagramsCalculation(hObject, eventdata, handles, results)
% This function will calcualte all the voronoi volumes & areas
data = get(gcf,'UserData');
options = data{2};

% Set the default flag for the waitbar and then look to see if the user
% passed a flag inside the eventdata
WB = 1;
if(~isempty(eventdata))
    WB = eventdata;
end

% Report the analysis
SpatialAnalysisMessage('Performing Voronoi domain analysis...')

% Get the (x,y,z) coordinates of all the points
p = results.points;
lenP = size(p,1);

% First call the voronoi function for n dimensions
% Voronoi vertices V and the Voronoi cells C
[V,C] = voronoin(p);
results.V = V;
results.C = C;

% Set a flag if the mex function pyramidarea exists
PYRAMIDAREAMEX = (exist('pyramidarea')==3);

% Go through each point
for a = 1:lenP
    if(any(a==results.ConvexHullIndices))
        % Then we are on a point on the convex hull so we do nothing
        results.Comment{a} = ['Point ',num2str(a,'%d'),' is on the convex hull'];
        results.NumVoronoiFacets(a) = NaN;
        results.NumVoronoiVertices(a) = NaN;
        results.VoronoiVolume(a) = NaN;
        results.VoronoiArea(a) = NaN;
        results.K{a} = [];
        results.VoronoiClass(a) = 4;
    else
        % Get the indices of the verticies of the current point
        indices = C{a};

        % Get the vertices for the voronoi diagram
        X = V(indices,:);

        % Check to see if the point is infected
        if(any(X(:,1)<options.MinX) || any(X(:,1)>options.MaxX) || ...
                any(X(:,2)<options.MinY) || any(X(:,2)>options.MaxY) || ...
                any(X(:,3)<options.MinZ) || any(X(:,3)>options.MaxZ))
            results.Comment{a} = ['Point ',num2str(a,'%d'),' is infected'];
            results.VoronoiClass(a) = 3;
        else
            results.VoronoiClass(a) = 1;
        end
        
        % Check to see if we have included any (inf inf inf) points by
        % accident
        if(any((isinf(X))))
            results.Comment{a} = ['Point ',num2str(a,'%d'),' is on the convex hull'];
            results.NumVoronoiFacets(a) = NaN;
            results.NumVoronoiVertices(a) = NaN;
            results.VoronoiVolume(a) = NaN;
            results.VoronoiArea(a) = NaN;
            results.K{a} = [];
            results.VoronoiClass(a) = 4;
        else
            % Proceed if we have not included any (inf inf inf) points
            try
                [K,Vol] = convhulln(X);
                
                % Calculate the area and the volume associated with each voronoi
                % side.  The sides are treated as pyramids to the center
                % point
                if(PYRAMIDAREAMEX)
                    % Need to subtract one from K since C code indices
                    % start at zero
                    [A,n] = pyramidarea(X,K-1,p(a,:));
                else
                    % Initialize the volume and area
                    KM = size(K,1);
                    A = zeros(KM,1);
                    n = zeros(KM,3);

                    % Calculate the area and direction vector locally
                    for b = 1:KM
                        [A(b),n(b,:)] = triarea3(X(K(b,:),1),X(K(b,:),2),...
                            X(K(b,:),3),p(a,:));
                    end
                end
                                       
                % Calculate the direction vector weighted by the area
                Vect1 = sum(repmat(A,1,3).*n);
                Vect2 = sum(abs(repmat(A,1,3).*n));
                
                
                % If all -x cancels +x similarly with y and z, then we
                % really can't compare the relative difference between x
                % and y, x and z, etc.  So, scale the result of Vec by the
                % relative absolute of x, y or z
                
                % Normalize the first vector
                norm1 = sqrt(sum(Vect1.^2));
                if(norm1>1)
                    Vect1 = Vect1./sqrt(sum(Vect1.^2));
                end
                
                % Check to see if the result is basically <0,0,0>
                if(all(abs(Vect1)<1e-5)) % That is all the values are close to zero
                    Vect2 = Vect2./max(Vect2);  % Make the largest value 1
                    Vect = Vect2 - min(Vect2); % Make the smallest values 0
                else
                    Vect = Vect1;
                end
                
                % Save all the data
                results.VoronoiDirVectors(a,:) = Vect;
                results.NumVoronoiVertices(a) = length(indices);
                results.NumVoronoiFacets(a) = size(K,1);
                results.VoronoiVolume(a) = Vol;
                results.VoronoiArea(a) = sum(A);
                results.K{a} = K;
                %results.VoronoiClass(a) = 1;
            catch
                % If there was an error in the convhulln then we can not
                % proceed until I know how to fix it
                [lastmsg,lastid] = lasterr;
                results.Comment{a} = ['Point ',num2str(a,'%d'),': ',...
                    lastmsg(1:min(find(lastmsg*1==10))-1)];
                results.NumVoronoiFacets(a) = NaN;
                results.NumVoronoiVertices(a) = NaN;
                results.VoronoiVolume(a) = NaN;
                results.VoronoiArea(a) = NaN;
                results.K{a} = [];
                results.VoronoiClass(a) = 5;
            end

        end

        % Update the waitbar - flush the graphics queue every 10%
        if(~mod(a,round(lenP/10)) && WB)
            SpatialAnalysisWaitBar(a/lenP);
        end
    end
end

% Now we need to know if any of the other points that are assumed to be of
% class 1 are double infected.  That is, one of their voronoi vertices is a
% vertex of an infected point, convex hull point, or "other" point

% Get the indices of all the vertices that belong to points that are not
% normal points
A = find(results.VoronoiClass~=1);

% Of these points, get all the vertices of the Voronoi domains
B = cell2mat(C(A)');

% Make a unique list
B = unique(reshape(B,numel(B),1));

% Go through the other points and see if they contain any vertices of the
% infected points.  If they do, then this point is a double infected point
for i = setdiff([1:length(results.VoronoiClass)],A)
    if(~isempty(intersect(C{i},B)))
        results.VoronoiClass(i) = 2;
    end
end

% Zero the waitbar
SpatialAnalysisWaitBar(0);
SpatialAnalysisMessageDONE
drawnow

% Set the calculation boolean to 1
results.VORONOICALC = 1;

% --------------------------------------------------------------------
function NearestNeighborMenu_Callback(hObject, eventdata, handles)
% This function will plot the nearest neighbors for each point
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Check to see if we have any points to do calculations on
if(length(results.X)==0)
    hED = errordlg('Not enough points to perform a Near Neighbor calculation',...
        '3D Spatial Analysis ERROR','modal');
    waitfor(hED)
    return
end

% Need to check if the points have been plotted, if not, plot them
if(~strcmp(PlotData.type,'NearestNeighbor'))
    PlotPoints(hObject, eventdata, handles,0)

    % No need to change the data table if it has already been changed
    if(get(handles.DataTable2Title,'value')~=2)
        set(handles.DataTable2Title,'value',2)
        DataTable2Title_Callback(hObject, eventdata, handles)
    end
end

% If we have not performed the nearest neighbor calculation yet, do that
% now
if(~results.NNCALC)
    results = NearestNeighborCalculation(hObject, eventdata, handles, results);
end

% Get the points and the current point
a = results.CurPoint;
p = results.points;

% Delete any previous items that need to be deleted
A = find(ishandle(PlotData.handles));
if(any(A))
    delete(PlotData.handles(A))
end

% Draw each nearest neighbor
SelectAxes(hObject, eventdata, handles, 'PlotAxes')
FixAxisLimits(hObject, eventdata, handles, 'PlotAxes')

hold on

% Check the NN Filter and see if we should render the point
CONTINUE = CheckNNFilter();

% Plot the nearest neighbors if we should CONTINUE
if(CONTINUE)
    % Get the indices of the near neighbors, ignore zero which is always
    % the first element since unique sorts the values it returns
    A = unique(results.NNIndices(a,:));
    for i = 2:numel(A)
        h(i-1) = plot3([p(a,1);p(A(i),1)],[p(a,2);p(A(i),2)],...
            [p(a,3);p(A(i),3)],'g.-','markersize',options.MarkerSize,...
            'color',[0 .5 0]);
    end

    % Find the minimum or true nearest neighbor to the color red
    set(h(find(A==results.NNIndex(a))-1),'color','r')

    % Draw the current point and make it black
    h(i) = plot3(p(a,1),p(a,2),p(a,3),'.k');
    
    % We used to make the dot larger as well
    %,'markersize',1.5*options.MarkerSize);
else
    % Draw the current point and make it black with a larger dot
    h(1) = plot3(p(a,1),p(a,2),p(a,3),'.k');
    
    % We used to make the dot larger as well
    %,'markersize',1.5*options.MarkerSize);
end

% Set the title of the plot
title('Near Neighbors')

% Update the PlotData
PlotData.handles = h;
PlotData.type = 'NearestNeighbor';
PlotData.style = '';

% Save the data
data{1} = results;
data{3} = PlotData;
set(gcf,'UserData',data)

% Update the Data table
UpdateDataTable(hObject, eventdata, handles)


% --------------------------------------------------------------------
function DelaunayMenu_Callback(hObject, eventdata, handles)
% This function will pot the delaunay tessellation diagrams
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Check to see if we have any points to do calculations on
if(length(results.X)==0)
    hED = errordlg('Not enough points to perform a Delaunay calculation',...
        '3D Spatial Analysis ERROR','modal');
    waitfor(hED)
    return
end

% Need to check if the points have been plotted, if not, plot them
if(~strcmp(PlotData.type,'Delaunay'))
    PlotPoints(hObject, eventdata, handles, 0)

    % No need to change the data table if it has already been changed
    if(get(handles.DataTable2Title,'value')~=3)
        set(handles.DataTable2Title,'value',3)
        DataTable2Title_Callback(hObject, eventdata, handles)
    end
end

% If we have not performed the nearest neighbor calculation yet, do that
% now
if(~results.NNCALC)
    results = NearestNeighborCalculation(hObject, eventdata, handles, results);
end

% Get the points and the current tetrahedron
a = results.CurTetra;
p = results.points;

% Delete any previous items that need to be deleted and initialize the
% handles 
A = find(ishandle(PlotData.handles));
if(any(A))
    delete(PlotData.handles(A))
end
h = [];


% Select the correct axis
SelectAxes(hObject, eventdata, handles, 'PlotAxes')
hold on

% Get the indices for the "current" delaunay tetrahedron
K = results.T(a,:);

% Check the NN Filter and see if we should render the point
CONTINUE = CheckDelaunayFilter();

if(CONTINUE)
    % Create a patch object (facet) for each Delaunay tetrahedron surface
    for b = 1:4
        K = circshift(K,[0 1]);
        h(b) = patch(p(K(1:3),1),p(K(1:3),2),p(K(1:3),3),'g');
    end

    % Set the transparency
    %alpha(h,options.Transparency)
end

% Plot the point we are currently on
%h(b+1) = plot3(p(a,1),p(a,2),p(a,3),'*r','markersize',options.MarkerSize);


drawnow

% Set the title for the plot
title('Delaunay Tessellation')

% Update the PlotData
PlotData.handles = h;
PlotData.type = 'Delaunay';
PlotData.style = '';

% Save the data
data{1} = results;
data{3} = PlotData;
set(gcf,'UserData',data)

% Update the Data table
UpdateDataTable(hObject, eventdata, handles)

% -------------------------------------------------------------------------
function PlotDelaunaySection(hObject, eventdata, handles)
% This function will plot the individual delaunay sections
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Need to check if the points have been plotted, if not, plot them
if(~strcmp(PlotData.type,'Delaunay'))
    PlotPoints(hObject, eventdata, handles)

    % No need to change the data table if it has already been changed
    if(get(handles.DataTable2Title,'value')~=2)
        set(handles.DataTable2Title,'value',2)
        DataTable2Title_Callback(hObject, eventdata, handles)
    end
end

% If we have not performed the nearest neighbor calculation yet, do that
% now
if(~results.NNCALC)
    results = NearestNeighborCalculation(hObject, eventdata, handles, results);
end

% Get the points and the current point, and the current slice
a = results.CurPoint;
p = results.points;
s = results.CurSection(a);

% Delete any previous items that need to be deleted
A = find(ishandle(PlotData.handles));
if(any(A))
    delete(PlotData.handles(A))
end

% Select the correct axis
SelectAxes(hObject, eventdata, handles, 'PlotAxes')
hold on

% Get the points and indices for the delaunay tessellation
K = results.DelaunayK{a};
X = results.DelaunayX{a};

% Create a surface for each voronoi surface

% Plot the outer facet
h(1) = patch(X(K(s,:),1),X(K(s,:),2),X(K(s,:),3),'g');

% Plot the 3 sides
h(2) = patch([X(K(s,1:2),1);p(a,1)],[X(K(s,1:2),2);p(a,2)],...
    [X(K(s,1:2),3);p(a,3)],'g');
h(3) = patch([X(K(s,1:3),1);p(a,1)],[X(K(s,1:3),2);p(a,2)],...
    [X(K(s,1:3),3);p(a,3)],'g');
h(4) = patch([X(K(s,2:3),1);p(a,1)],[X(K(s,2:3),2);p(a,2)],...
    [X(K(s,2:3),3);p(a,3)],'g');


% Set the transparency
%alpha(h,options.Transparency)

% Plot the point we are currently on
h(5) = plot3(p(a,1),p(a,2),p(a,3),'*r','markersize',options.MarkerSize);
drawnow

% Set the title for the plot
title('Delaunay Tessellation')

% Update the PlotData
PlotData.handles = h;
PlotData.type = 'Delaunay';
PlotData.style = '';

% Save the data
data{1} = results;
data{3} = PlotData;
set(gcf,'UserData',data)

%--------------------------------------------------------------------------
function results = NearestNeighborCalculation(hObject, eventdata, handles, results)
% This function will calcualte all the nearest neighbor distances
data = get(gcf,'UserData');
results = data{1};
options = data{2};

% Report the analysis
SpatialAnalysisMessage('Performing Delaunay/neighbor analysis...')

% Calculate the near neighbor distances and the delaunay statistics
[results.MinNNDist,results.MaxNNDist,results.AvgNNDist,results.NumNN,...
    results.NNIndices,results.NNIndex,results.T,results.DelaunaySegLen,...
    results.DelaunayArea,results.DelaunayTotalArea,results.DelaunayVolume,...
    results.SegLenIndex,results.FacetIndex,results.DelaunayNumFacets] = ...
    nearestneighbor3(results.points);
    
% Classify the points and store the classification value in NNClass.  This
% could change if the user changes the boundary position
results.NNClass = filternnpoints(results.points,results.MinNNDist,...
    results.NNIndices,options.MinX,options.MaxX,options.MinY,...
    options.MaxY,options.MinZ,options.MaxZ);

% Zero the waitbar
SpatialAnalysisMessageDONE

% Set the calculation boolean to 1
results.NNCALC = 1;

% Save the data
data{1} = results;
set(gcf,'UserData',data)

% --------------------------------------------------------------------
function varargout = AutocorrelationMenu_Callback(hObject, eventdata, handles)
% This function will perform an autocorrelation on the data
data = get(gcf,'Userdata');
results = data{1};
options = data{2};
PlotData = data{3};

% Check to see if we have any points to do calculations on
if(length(results.X)==0)
    hED = errordlg('Not enough points to perform an auto correlation',...
        '3D Spatial Analysis ERROR','modal');
    waitfor(hED)
    return
end

% Set some of the options
options.WaitBar = @SpatialAnalysisWaitBar; %handles.WaitBar;
options.AxesHandle = handles.AutocorrelationAxes;
options.EdgeCorrection = options.EdgeCorrectionAC;
options.Filter = options.FilterAC;
options.MaxRadius = options.MaxRadiusAC;
options.NumBins = options.NumACBins;
options.MarkerSize = options.MarkerSizeAC;

% Set the Data Table to the autocorrelation
if(get(handles.DataTable2Title,'value')~=5)
    set(handles.DataTable2Title,'value',5)
    DataTable2Title_Callback(hObject, eventdata, handles)
end

% Check to see if we have performed the autocorrelation already
if(~results.ACCALC)
    if(options.PLOTAC)
        % Select the right axes and then report the anlaysis
        SelectAxes(hObject, eventdata, handles, 'AutocorrelationAxes')
        hold off
    end    
    SpatialAnalysisMessage('Performing autocorrelation analysis...')
    [results.ACBins,results.ACCount,results.ACBinVolume,...
        results.SampleVolume,results.ACIndices,ACStats,results.ACx,...
        results.ACy,results.ACz] = autocorrelation3(results.points,options);
    SpatialAnalysisMessageDONE
        
    % Check to see if we had an error in the autocorrelation
    if(ischar(results.ACBins))
        errordlg(results.ACCount,'Autocorrelation ERROR','modal')
        return
    end

    % Strip out the autocorrelation statistics
    results.ACDensity = ACStats.density;
    results.ACDc = ACStats.Dc;
    results.ACreff = ACStats.reff;
    results.ACrm = ACStats.rm;
    results.ACk = ACStats.k;
    results.ACp = ACStats.p;
    expon = round(log10(results.ACDensity*ACStats.len)); 
    set(gca,'UserData',...
        ['(x10^{',num2str(expon),'} ',options.Units,'^{-3})']);
    
    results.ACCALC = 1;
else
    % If we've already calculated the AC then just show the plot
    if(options.PLOTAC)
        SelectAxes(hObject, eventdata, handles, 'AutocorrelationAxes')
        hCB = colorbar;
    end
end

% Update the PlotData if the user asked to plot the autocorrelation
if(options.PLOTAC)
    % Update the PlotData
    PlotData.handles = [];
    PlotData.type = 'Autocorrelation';
    PrintAxisLabels(hObject, eventdata, handles)
    FixAxisLimits(hObject, eventdata, handles,'AutocorrelationAxes')
    title('3D Autocorrelation')
    hold off
end

% Save the data
data{1} = results;
data{3} = PlotData;
set(gcf,'UserData',data)

% If the user asked for an output, then pass back the results structure
if(nargout==1 | handles.DataTable2Title==hObject)
    varargout{1} = results;
    return
end

% Call the AutocorrelationHistogram
if(options.PLOTAC)
    AutocorrelationHistogram_Callback(hObject, eventdata, handles)
end

% --------------------------------------------------------------------
function AutocorrelationMovieMenu_Callback(hObject, eventdata, handles)
% call the autocorrelation menu function
AutocorrelationMenu_Callback(hObject, eventdata, handles)
% Call the send to new figure menu
hF = SendFigureMenu_Callback(hObject, eventdata, handles);

% Make the new figure current
figure(hF)

% Make the axis current
axes(get(hF,'children'))

% Hide the axis
set(gca,'visible','off')

axis vis3d
zoom(1.5)
colorbar
EL = 30;
j = 0;
for AZ = -37.5:-3:-397.5
    j = j+1;
    view(AZ,EL)
    M(j) = getframe(hF);
end

EL = 30:-1:0
AZ = -37.5:(-90+37.5)/(length(EL)-1):-90
for i = 1:length(EL)
    j = j+1;
    view(AZ(i),EL(i))
    M(j) = getframe(hF);
end

% Ask the user the name of the AVI file to create
[filename,pathname,filterindex] = uiputfile('*.avi','Save movie file as...');

% If the user typed in a correct filename
if(filterindex)
    movie2avi(M,[pathname,filename],'compression','none','videoname',...
        '3D Autocorrelation')
end

%---------------------------------------------------------------------
% PLOT MENU

% --------------------------------------------------------------------
function PlotPoints(hObject, eventdata, handles, varargin)
% This function will plot the points in the figure window
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Switch to the General data table
% No need to change the data table if it has already been changed
if(get(handles.DataTable2Title,'value')~=1 & hObject==handles.DataPointsMenu)
    set(handles.DataTable2Title,'value',1)
    DataTable2Title_Callback(hObject, eventdata, handles)
end

% Get the update (data table) flag if the user passed one
if(length(varargin))
    UPDATE = varargin{1};
else
    UPDATE = 1;
end

% Delete any previous items that need to be deleted
SelectAxes(hObject, eventdata, handles, 'PlotAxes')
FixAxisLimits(hObject, eventdata, handles, 'PlotAxes')

A = find(ishandle(PlotData.handles));
if(any(A))
    delete(PlotData.handles(A))
    delete(get(gca,'children'))
else
    delete(get(gca,'children'))
end

% Get the the data points
p = results.points;

% Plot the points
hP = plot3(p(:,1),p(:,2),p(:,3),'.','markersize',options.MarkerSize);

set(gca,'Tag','PlotAxes','Interruptible','off','ButtonDownFcn',...
    'SpatialAnalysis(''ButtonDownPlotAxes'',gcbo,[],guidata(gcbo))')
set(hP,'Interruptible','off','ButtonDownFcn',...
    'SpatialAnalysis(''SelectPoint'',gcbo,[],guidata(gcbo))')

% Set the view, print the axis lables, clear the title, set the axis
% limits, turn the box frame on and the colorbar off
%view(3)
PrintAxisLabels(hObject, eventdata, handles)
title('')
box on
colorbar off
hold on

% Save the plot data
PlotData.handles = hP;
PlotData.type = 'PlotPoints';
PlotData.style = '';

data{3} = PlotData;
set(gcf,'UserData',data)

% Update the Data table
if(UPDATE)
    UpdateDataTable(hObject, eventdata, handles)
end

%--------------------------------------------------------------------------
function PlotFalseColor(hObject, eventdata, handles, results, D)
% This function does the plotting of the false colorplot
data = get(gcf,'UserData');
options = data{2};
PlotData = data{3};

% Delete any previous items that need to be deleted
A = find(ishandle(PlotData.handles));
if(any(A))
    delete(PlotData.handles(A))
end

% Plot the areas are dots with their size and color representative of the
% measured area
SelectAxes(hObject, eventdata, handles, 'PlotAxes')
hold off
A = find(D<=0);
D(A) = eps;
scaleddata = D/max(D)*100;
h = scatter3('v6',results.X,results.Y,results.Z,scaleddata,scaleddata,'filled');

% Load the marker size data into the userdata and set the current point
% markersize
for i = 1:length(h)
    set(h(i),'UserData',get(h(i),'Markersize'),'edgecolor',[0 0 0])
end
set(h(results.CurPoint),'markersize',15)

% Turn the box frame on, set the view and the axis to equal, print the axis
% labels, set the title, set the axis limits
box on
view(3)
PrintAxisLabels(hObject, eventdata, handles)
FixAxisLimits(hObject, eventdata, handles, 'PlotAxes')
set(gca,'clim',[0 100],'Tag','PlotAxes','Interruptible','off',...
    'ButtonDownFcn','SpatialAnalysis(''ButtonDownPlotAxes'',gcbo,[],guidata(gcbo))',...
    'UserData',max(D))

% Turn the colorbar on and set the tick labels
FixColorbarAxisLabels(hObject, eventdata, handles)

% Update the PlotData
PlotData.handles = h;
PlotData.type = 'FalseColor';
PlotData.style = '';

% Update the data
data{1} = results;
data{3} = PlotData;
set(gcf,'UserData',data)

% Update the Data table
UpdateDataTable(hObject, eventdata, handles)

% --------------------------------------------------------------------
function FalseColorVolumeMenu_Callback(hObject, eventdata, handles)
% This function will plot the data points with colors of eat dot and the
% size of each dot proportional to the volume
data = get(gcf,'UserData');
results = data{1};

% Switch to the General data table
% No need to change the data table if it has already been changed
if(get(handles.DataTable2Title,'value')~=1)
    set(handles.DataTable2Title,'value',1)
    DataTable2Title_Callback(hObject, eventdata, handles)
end

% Plot the false color plot
PlotFalseColor(hObject,eventdata,handles,results,results.Volume)
title(handles.PlotAxes,'Point Volume')

% --------------------------------------------------------------------
function FalseColorAreaMenu_Callback(hObject, eventdata, handles)
% This function will plot the data points with colors of each dot and the
% size of each dot proportional to the area
data = get(gcf,'UserData');
results = data{1};

% Switch to the General data table
% No need to change the data table if it has already been changed
if(get(handles.DataTable2Title,'value')~=1)
    set(handles.DataTable2Title,'value',1)
    DataTable2Title_Callback(hObject, eventdata, handles)
end

% Plot the false color plot
PlotFalseColor(hObject,eventdata,handles,results,results.Area)
title('Point Area')


% --------------------------------------------------------------------
function FalseColorEccentricityMenu_Callback(hObject, eventdata, handles)
% This function will plot the data points with colors of each dot and the
% size of each dot proportional to the Eccentricity
data = get(gcf,'UserData');
results = data{1};

% Switch to the General data table
% No need to change the data table if it has already been changed
if(get(handles.DataTable2Title,'value')~=1)
    set(handles.DataTable2Title,'value',1)
    DataTable2Title_Callback(hObject, eventdata, handles)
end

% Plot the false color plot
PlotFalseColor(hObject,eventdata,handles,results,results.Eccentricity)
title(handles.PlotAxes,'Eccentricity')


% --------------------------------------------------------------------
function FalseColorCircularityMenu_Callback(hObject, eventdata, handles)
% This function will plot the data points with colors of each dot and the
% size of each dot proportional to the Circularity
data = get(gcf,'UserData');
results = data{1};

% Switch to the General data table
% No need to change the data table if it has already been changed
if(get(handles.DataTable2Title,'value')~=1)
    set(handles.DataTable2Title,'value',1)
    DataTable2Title_Callback(hObject, eventdata, handles)
end

% Plot the false color plot
PlotFalseColor(hObject,eventdata,handles,results,results.Circularity)
title('Circularity')


% --------------------------------------------------------------------
function FalseColorSphericityMenu_Callback(hObject, eventdata, handles)
% This function will plot the data points with colors of each dot and the
% size of each dot proportional to the Sphericity
data = get(gcf,'UserData');
results = data{1};

% Switch to the General data table
% No need to change the data table if it has already been changed
if(get(handles.DataTable2Title,'value')~=1)
    set(handles.DataTable2Title,'value',1)
    DataTable2Title_Callback(hObject, eventdata, handles)
end

% Plot the false color plot
PlotFalseColor(hObject,eventdata,handles,results,results.Sphericity)
title(handles.PlotAxes,'Sphericity')

% -------------------------------------------------------------------------
function FalseColorVoronoiVolMenu_Callback(hObject, eventdata, handles)
% This function will plot the data points with the color and size of each 
% dot proportional to the voronoi volume
data = get(gcf,'UserData');
results = data{1};
options = data{2};

% Switch to the Voronoi data table
% No need to change the data table if it has already been changed
if(get(handles.DataTable2Title,'value')~=4)
    set(handles.DataTable2Title,'value',4)
    DataTable2Title_Callback(hObject, eventdata, handles)
end

% Get the Voronoi Indices appropriate for the filter
A = GetFilteredVoronoiIndices(hObject, eventdata, handles, results);

% Plot the false color plot
D = eps*ones(results.NumPoints,1);
D(A) = results.VoronoiVolume(A);
PlotFalseColor(hObject,eventdata,handles,results,D)
title(handles.PlotAxes,'Voronoi Volume')

% --------------------------------------------------------------------
function FalseColorNNDist_Callback(hObject, eventdata, handles)
% This function will plot the data points with the color and size of each 
% dot proportional to the minimum nearest neighbor distance
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Switch to the Voronoi data table
% No need to change the data table if it has already been changed
if(get(handles.DataTable2Title,'value')~=2)
    set(handles.DataTable2Title,'value',2)
    DataTable2Title_Callback(hObject, eventdata, handles)
end

% Get the proper NN indices based on the filter
A = GetFilteredNNIndices(hObject, eventdata, handles, results);

% Plot the false color plot
D = eps*ones(results.NumPoints,1);
D(A) = results.MinNNDist(A);
PlotFalseColor(hObject,eventdata,handles,results,D)
title(handles.PlotAxes,'Nearest Neighbor Distance')

% --------------------------------------------------------------------
function FalseColorMaxNNDist_Callback(hObject, eventdata, handles)
% This function will plot the data points with the color and size of each 
% dot proportional to the maximum neighbor distance
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Switch to the Voronoi data table
% No need to change the data table if it has already been changed
if(get(handles.DataTable2Title,'value')~=2)
    set(handles.DataTable2Title,'value',2)
    DataTable2Title_Callback(hObject, eventdata, handles)
end

% Get the proper NN indices based on the filter
A = GetFilteredNNIndices(hObject, eventdata, handles, results);

% Plot the false color plot
D = eps*ones(results.NumPoints,1);
D(A) = results.MaxNNDist(A);
PlotFalseColor(hObject,eventdata,handles,results,D)
title(handles.PlotAxes,'Maximum Neighbor Distance')


% --------------------------------------------------------------------
function FalseColorAvgNNDistMenu_Callback(hObject, eventdata, handles)
% This function will plot the data points with the color and size of each 
% dot proportional to the average nearest neighbor distance
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Switch to the Voronoi data table
% No need to change the data table if it has already been changed
if(get(handles.DataTable2Title,'value')~=2)
    set(handles.DataTable2Title,'value',2)
    DataTable2Title_Callback(hObject, eventdata, handles)
end

% Get the proper NN indices based on the filter
A = GetFilteredNNIndices(hObject, eventdata, handles, results);

% Plot the false color plot
D = eps*ones(results.NumPoints,1);
D(A) = results.AvgNNDist(A);
PlotFalseColor(hObject,eventdata,handles,results,D)
title(handles.PlotAxes,'Average Delaunay Segment Length')

% -------------------------------------------------------------------------
function FalseColorVoronoiAreaMenu_Callback(hObject, eventdata, handles)
% This function will plot the data points with the color and size of each 
% dot proportional to the voronoi area
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Switch to the Voronoi data table
% No need to change the data table if it has already been changed
if(get(handles.DataTable2Title,'value')~=4)
    set(handles.DataTable2Title,'value',4)
    DataTable2Title_Callback(hObject, eventdata, handles)
end

% Get the Voronoi Indices appropriate for the filter
A = GetFilteredVoronoiIndices(hObject, eventdata, handles, results);

% Plot the false color plot
D = eps*ones(results.NumPoints,1);
D(A) = results.VoronoiArea(A);
PlotFalseColor(hObject,eventdata,handles,results,D)
title(handles.PlotAxes,'Voronoi Area')

% --------------------------------------------------------------------
function NNHistogramMenu_Callback(hObject, eventdata, handles)
% This function will plot the minimum nearest neighbor statistical data in 
% a histogram form
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Need to check if the points have been plotted, if not, plot them
if(~strcmp(PlotData.type,'PlotPoints'))
    PlotPoints(hObject, eventdata, handles,0)
end

% Calculate the nearest neighbor histogram statistics
results = NNHistogramCalc(hObject, eventdata, handles, results);

% See if we have any points to do the plotting with
if(isempty(results.NNBinCount))
    % Give a warning
    msg = ['No nearest neighbor points are available under the current filter: ',...
        options.NNFilter];
    hW = warndlg(msg,'3D Spatial Analysis-Delaunay Calculation','modal');
    uiwait(hW)
    SpatialAnalysisMessage(msg)
else
    % Plot the nearest neighbor histogram
    [results,PlotData] = PlotNNHistogram(hObject, eventdata, handles, ...
        results, PlotData);
end

% Save the results.
data{1} = results;
data{3} = PlotData;
set(gcbf,'UserData',data)

% --------------------------------------------------------------------
function A = GetFilteredNNIndices(hObject, eventdata, handles, results)
% This function will return the near neighbor indices appropriate for the
% given filter
data = get(gcf,'UserData');
options = data{2};

% 1 is a regular point, 2 is a double infected point,
% 3 is an infected point, 4 is the convex hull
switch(options.NNFilter)
    case 'None'
        A = 1:length(results.NNClass);
    case 'Convex Hull',
        A = find(results.NNClass<4);
    case 'Infected Points'
        A = find(results.NNClass<3);
    case 'Double Infected Points'
        A = find(results.NNClass==1);
end

%--------------------------------------------------------------------------
function A = GetFilteredVoronoiIndices(hObject, eventdata, handles, results)
% This function will return the voronoi indices appropriate for the
% given filter
data = get(gcf,'UserData');
options = data{2};

% 1 is a regular point, 2 is double infected, 3 is infected
% 4 is the convex hull, , 5 is other problem 
switch(options.VoronoiFilter)
    case 'Convex Hull'
        A = find(results.VoronoiClass<4);
    case 'Infected Points'
        A = find(results.VoronoiClass<3);
    case 'Double Infected Points'
        A = find(results.VoronoiClass==1);        
end

%--------------------------------------------------------------------------
function results = NNHistogramCalc(hObject, eventdata, handles, results)
% This function will determine the bin sizes and calculate the counts per
% bin for the nearest neighbor histogram
data = get(gcf,'UserData');
options = data{2};

% Get the proper NN indices based on the filter
A = GetFilteredNNIndices(hObject, eventdata, handles, results);

% Get all the nearest neighbor distances, calculate the bin edges, then
% perform the histogram calculation.
results.NNDistHist = results.MinNNDist(A);
results.NNBinEdges = [0:options.NNBinSize:options.NNBinSize*options.NumNNBins,inf];

% Check to see if we have any points to do the calculation on.
if(isempty(A))
    results.NNBinCount = [];
    results.NNBins = [];
else
    [results.NNBinCount,results.NNBins] = histc(results.NNDistHist,results.NNBinEdges);
    results.NNBinEdges(length(results.NNBinEdges)) = ...
        results.NNBinEdges(length(results.NNBinEdges)-1)+options.NNBinSize;
end

% Don't save the results structure, just pass it bask.  The calling
% function will have a copy of results already loaded, so saving the
% results structure here will not create a change in the calling
% functions's version.

%--------------------------------------------------------------------------
function [results,PlotData] = PlotNNHistogram(hObject, eventdata, handles, ...
    results, PlotData)
% This function will plot the nearest neighbor histogram

% Get the options from the figure window
data = get(gcf,'UserData');
options = data{2};

% Make the datatable invisible and make the histogram axes visible
SelectHistogramDataPanel(hObject, eventdata, handles, 'Histogram')

% Plot the frequency as a histogram
NNBinEdges = results.NNBinEdges;
freq = results.NNBinCount/sum(results.NNBinCount);
for i = 1:length(NNBinEdges)-1
    hB(i) = fill([NNBinEdges(i) NNBinEdges(i) NNBinEdges(i+1) NNBinEdges(i+1)],...
        [0 freq(i) freq(i) 0],'b');
    hold on
end
hold off
set(gca,'xlim',[min(NNBinEdges),max(NNBinEdges)],'ButtonDownFcn',...
    'SpatialAnalysis(''ButtonDownPlotAxes'',gcbo,[],guidata(gcbo))',...
    'DeleteFcn','SpatialAnalysis(''DeleteHistogramAxes'',gcbo,[],guidata(gcbf))')
xlabel(['Distance (',options.Units,')'])
ylabel('Frequency (%)')
title('Nearest Neighbor Distance')

% Calculate the average, standard deviation, min and max of the population
NNavg = mean(results.NNDistHist);
NNstd = std(results.NNDistHist);
minNN = min(results.NNDistHist);
maxNN = max(results.NNDistHist);

% Print these values on the histogram as a legend
hB(i+1) = text(.98,.99,...
    {['Avg=',num2str(NNavg,'%4.2f'),', \sigma=',num2str(NNstd,'%4.2f')];...
    ['min=',num2str(minNN,'%4.2f'),', max=',num2str(maxNN,'%4.2f'),...
    ' (',options.Units,')']},...
    'VerticalAlignment','top','HorizontalAlignment','right',...
    'EdgeColor',[0 0 0],'units','normalized');

% Pass the PlotData back so that it can be saved
PlotData.handles = hB;
PlotData.type = 'Histogram';
PlotData.style = 'NearestNeighbor';

% --------------------------------------------------------------------
function DelaunaySegmentHistogramMenu_Callback(hObject, eventdata, handles)
% This function will plot the nearest neighbor statistical data in a
% histogram form
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Need to check if the points have been plotted, if not, plot them
if(~strcmp(PlotData.type,'PlotPoints'))
    PlotPoints(hObject, eventdata, handles,0)
end

% Calculate the histogram statistics
results = DelaunayHistogramCalc(hObject, eventdata, handles, results);

% Check to see if we have any data from the histogram calculation
if(isempty(results.DelaunaySegLenBinCount))
    msg = ['No Delaunay segments are available under the current filter: ',...
        options.NNFilter];
    hW = warndlg(msg,'3D Spatial Analysis-Delaunay Calculation','modal');
    uiwait(hW)
    SpatialAnalysisMessage(msg)
    return
end

% Plot the Filtered Delaunay Segment Lengths in a histogram
[results,PlotData] = PlotDelaunaySegHistogram(hObject, eventdata, handles,...
    results, PlotData);

% Save the results.
data{1} = results;
data{3} = PlotData;
set(gcbf,'UserData',data)


%--------------------------------------------------------------------------
function results = DelaunayHistogramCalc(hObject, eventdata, handles, results)
% This function will calculate all the delaunay segment statistics
data = get(gcf,'UserData');
options = data{2};

% Get the proper Delaunay indices for the segment length, area, and volume
[B1,B2,B3] = GetFilteredDelaunayIndices(hObject, eventdata, handles, results, flag);

% Make sure we have some points to do the calculation on
if(isempty(B1))
    results.DelaunaySegLenHistIndex = [];
    results.DelaunaySegLenBinEdges = ...
        [0:options.DelaunaySegLenBinSize:options.DelaunaySegLenBinSize*options.NumDelaunaySegLenBins,inf];
    results.DelaunaySegLenBinCount = [];
    results.DelaunaySegLenBins = [];
else
    % Calculate the bin edges, then perform the histogram calculation with the
    % filtered set of Delaunay segment lengths "B1"
    results.DelaunaySegLenHistIndex = B1;
    results.DelaunaySegLenBinEdges = ...
        [0:options.DelaunaySegLenBinSize:options.DelaunaySegLenBinSize*options.NumDelaunaySegLenBins,inf];
    [results.DelaunaySegLenBinCount,results.DelaunaySegLenBins] = histc(results.DelaunaySegLen(B1),...
        results.DelaunaySegLenBinEdges);
    results.DelaunaySegLenBinEdges(length(results.DelaunaySegLenBinEdges)) = ...
        results.DelaunaySegLenBinEdges(length(results.DelaunaySegLenBinEdges)-1)+options.NNBinSize;
end

% Make sure we have some points to do the calculation on
if(isempty(B2))
    results.DelaunayAreaHistIndex = [];
    results.DelaunayAreaBinEdges = ...
        [0:options.DelaunayAreaBinSize:options.DelaunayAreaBinSize*options.NumDelaunayAreaBins,inf];
    results.DelaunayAreaBinCount = [];
    results.DelaunayAreaBins = []; 
else
    % Calculate the bin edges, then perform the histogram calculation with the
    % filtered set of Delaunay areas "B2"
    results.DelaunayAreaHistIndex = B2;
    results.DelaunayAreaBinEdges = ...
        [0:options.DelaunayAreaBinSize:options.DelaunayAreaBinSize*options.NumDelaunayAreaBins,inf];
    [results.DelaunayAreaBinCount,results.DelaunayAreaBins] =...
        histc(results.DelaunayArea(B2),results.DelaunayAreaBinEdges);
    results.DelaunayAreaBinEdges(length(results.DelaunayAreaBinEdges)) = ...
        results.DelaunayAreaBinEdges(end-1)+options.DelaunayAreaBinSize;
end

% Make sure we have some points to do the calculation on
if(isempty(B3))
    results.DelaunayVolHistIndex = [];
    results.DelaunayVolBinEdges = ...
        [0:options.DelaunayVolBinSize:options.DelaunayVolBinSize*options.NumDelaunayVolBins,inf];
    results.DelaunayVolBinCount = [];
    results.DelaunayVolBins = [];
    
else
    % Calculate the bin edges, then perform the histogram calculation on with
    % the filtered set of Delanay volumes "B3"
    results.DelaunayVolHistIndex = B3;
    results.DelaunayVolBinEdges = ...
        [0:options.DelaunayVolBinSize:options.DelaunayVolBinSize*options.NumDelaunayVolBins,inf];
    [results.DelaunayVolBinCount,results.DelaunayVolBins] =...
        histc(results.DelaunayVolume(B3),results.DelaunayVolBinEdges);
    results.DelaunayVolBinEdges(length(results.DelaunayVolBinEdges)) = ...
        results.DelaunayVolBinEdges(end-1)+options.DelaunayVolBinSize;
end

%--------------------------------------------------------------------------
function varargout = GetFilteredDelaunayIndices(hObject, eventdata, handles, results, flag);
% This function will return the indices (rows) for the Delaunay analysis of
% interest.  Pass as the last argument either 'Seg','Area', or 'Vol' and
% the function will return the correct indices for that analysis

% Get the proper NN indices based on the filter
A = GetFilteredNNIndices(hObject, eventdata, handles, results);

if(nargout>1)
    % If the user asked for more than one output variable, then pass back
    % all 3 sets of indices
    SegLenIndexFlag = ismember(results.SegLenIndex,A);
    varargout{1} = find(all(SegLenIndexFlag')');

    FacetIndexFlag = ismember(results.FacetIndex,A);
    varargout{2} = find(all(FacetIndexFlag')');

    VolIndexFlag = ismember(results.T,A);
    varargout{3} = find(all(VolIndexFlag')');
else
    % Otherwise, only pass back the one asked for by the last input
    % argument
    switch(lower(flag))
        case {'seg','segment'}
            SegLenIndexFlag = ismember(results.SegLenIndex,A);
            varargout{1} = find(all(SegLenIndexFlag')');
        case 'area'
            FacetIndexFlag = ismember(results.FacetIndex,A);
            varargout{1} = find(all(FacetIndexFlag')');
        case {'vol','volume'}
            VolIndexFlag = ismember(results.T,A);
            varargout{1} = find(all(VolIndexFlag')');

    end
end


%--------------------------------------------------------------------------
function [results,PlotData] = PlotDelaunaySegHistogram(hObject, eventdata,...
    handles, results, PlotData)
% This function will plot the histogram for the filtered delaunay segment
% lengths.

% Get the options data
data = get(gcf,'UserData');
options = data{2};

% Make the datatable invisible and make the histogram axes visible
SelectHistogramDataPanel(hObject, eventdata, handles, 'Histogram')

% Plot the frequency as a histogram
NNBinEdges = results.DelaunaySegLenBinEdges;
freq = results.DelaunaySegLenBinCount/sum(results.DelaunaySegLenBinCount);
for i = 1:length(NNBinEdges)-1
    hB(i) = fill([NNBinEdges(i) NNBinEdges(i) NNBinEdges(i+1) NNBinEdges(i+1)],...
        [0 freq(i) freq(i) 0],'b');
    hold on
end
hold off
set(gca,'xlim',[min(NNBinEdges),max(NNBinEdges)],'ButtonDownFcn',...
    'SpatialAnalysis(''ButtonDownPlotAxes'',gcbo,[],guidata(gcbo))',...
    'DeleteFcn','SpatialAnalysis(''DeleteHistogramAxes'',gcbo,[],guidata(gcbf))')
xlabel(['Distance (',options.Units,')'])
ylabel('Frequency (%)')
title('Delaunay Segment Length')

% Calculate the average, standard deviation, min and max of the population
NNavg = mean(results.DelaunaySegLen(results.DelaunaySegLenHistIndex));
NNstd = std(results.DelaunaySegLen(results.DelaunaySegLenHistIndex));
minNN = min(results.DelaunaySegLen(results.DelaunaySegLenHistIndex));
maxNN = max(results.DelaunaySegLen(results.DelaunaySegLenHistIndex));

% Print these values on the histogram
hB(i+1) = text(.98,.99,...
    {['Avg=',num2str(NNavg,'%4.2f'),', \sigma=',num2str(NNstd,'%4.2f')];...
    ['min=',num2str(minNN,'%4.2f'),', max=',num2str(maxNN,'%4.2f'),...
    ' (',options.Units,')']},...
    'VerticalAlignment','top','HorizontalAlignment','right',...
    'EdgeColor',[0 0 0],'units','normalized');

% Pass the PlotData back so that it can be saved
PlotData.handles = hB;
PlotData.type = 'Histogram';
PlotData.style = 'DelaunaySegLen';

% --------------------------------------------------------------------
function DelaunayVolHistogramMenu_Callback(hObject, eventdata, handles)
% This function is called when the user asks to plot the Delaunay 
% tetrahedron volumes in histogram form.  
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Need to check if the points have been plotted, if not, plot them
if(~strcmp(PlotData.type,'PlotPoints'))
    PlotPoints(hObject, eventdata, handles,0)
    colorbar off
end

% Calculate the histogram statistics
results = DelaunayHistogramCalc(hObject, eventdata, handles, results);

% Check to see if we have any data from the histogram calculation
if(isempty(results.DelaunayVolBinCount))
    msg = ['No Delaunay tetrahedra are available under the current filter: ',...
        options.NNFilter];
    hW = warndlg(msg,'3D Spatial Analysis-Delaunay Calculation','modal');
    uiwait(hW)
    SpatialAnalysisMessage(msg)
    return
end

% Plot the filtered Delaunay Volume historam
[results,PlotData] = PlotDelaunayVolHistogram(hObject, eventdata, handles,...
    results, PlotData);

% Save the results.
data{1} = results;
data{3} = PlotData;
set(gcbf,'UserData',data)

%--------------------------------------------------------------------------
function [results,PlotData] = PlotDelaunayVolHistogram(hObject, eventdata,...
    handles, results, PlotData)
% This function will plot the histogram for the filtered delaunay volumes.

% Get the options data
data = get(gcf,'UserData');
options = data{2};

% Make the datatable invisible and make the histogram axes visible
SelectHistogramDataPanel(hObject, eventdata, handles, 'Histogram')

% Find the multiplying factor
expon = floor(log10(max(results.DelaunayVolBinEdges)));
DelaunayVolBinEdges = results.DelaunayVolBinEdges/10^expon;

% Plot the bins as filled 2D polygons
freq = results.DelaunayVolBinCount/sum(results.DelaunayVolBinCount);
for i = 1:length(DelaunayVolBinEdges)-1
    hB(i) = fill([DelaunayVolBinEdges(i) DelaunayVolBinEdges(i)...
        DelaunayVolBinEdges(i+1) DelaunayVolBinEdges(i+1)],...
        [0 freq(i) freq(i) 0],'b');
    hold on
end
hold off
set(gca,'xlim',[min(DelaunayVolBinEdges),max(DelaunayVolBinEdges)],...
    'ButtonDownFcn','SpatialAnalysis(''ButtonDownPlotAxes'',gcbo,[],guidata(gcbo))',...
    'DeleteFcn','SpatialAnalysis(''DeleteHistogramAxes'',gcbo,[],guidata(gcbf))')
xlabel(['Volume (x10^',num2str(expon),' ',options.Units,'^3)'])
ylabel('Frequency (%)')
title('Delaunay Tetrahedra Volume')

% Calculate the average, standard deviation, min and max of the population
NNavg = mean(results.DelaunayVolume(results.DelaunayVolHistIndex))/10^expon;
NNstd = std(results.DelaunayVolume(results.DelaunayVolHistIndex))/10^expon;
minNN = min(results.DelaunayVolume(results.DelaunayVolHistIndex))/10^expon;
maxNN = max(results.DelaunayVolume(results.DelaunayVolHistIndex))/10^expon;

% Print these values on the histogram
hB(i+1) = text(.98,.99,...
    {['Avg=',num2str(NNavg,'%4.2f'),', \sigma=',num2str(NNstd,'%4.2f'),...
    ', min=',num2str(minNN,'%4.2f')];['max=',num2str(maxNN,'%4.2f'),...
    ' (x10^',num2str(expon),' ',options.Units,'^3)']},...
    'VerticalAlignment','top','HorizontalAlignment','right',...
    'EdgeColor',[0 0 0],'units','normalized');

% Save the plot data information
PlotData.handles = hB;
PlotData.type = 'Histogram';
PlotData.style = 'DelaunayVolume';

% --------------------------------------------------------------------
function DelaunayAreaHistogramMenu_Callback(hObject, eventdata, handles)
% This function will plot the Delaunay Area for all the pyramids in the 
% sample space in a histogram form
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Need to check if the points have been plotted, if not, plot them
if(~strcmp(PlotData.type,'PlotPoints'))
    PlotPoints(hObject, eventdata, handles,0)
    colorbar off
end

% Calculate the histogram statistics
results = DelaunayHistogramCalc(hObject, eventdata, handles, results);

% Check to see if we have any data from the histogram calculation
if(isempty(results.DelaunayAreaBinCount))
    msg = ['No Delaunay facets are available under the current filter: ',...
        options.NNFilter];
    hW = warndlg(msg,'3D Spatial Analysis-Delaunay Calculation','modal');
    uiwait(hW)
    SpatialAnalysisMessage(msg)
    return
end

% Plot the filtered Delaunay facet surface area historam
[results,PlotData] = PlotDelaunayAreaHistogram(hObject, eventdata, handles,...
    results, PlotData);

% Save the results.
data{1} = results;
data{3} = PlotData;
set(gcbf,'UserData',data)


%--------------------------------------------------------------------------
function [results,PlotData] = PlotDelaunayAreaHistogram(hObject, eventdata,...
    handles, results, PlotData)
% This function will plot the histogram for the filtered delaunay facet
% areas

% Get the options data
data = get(gcf,'UserData');
options = data{2};

% Make the datapanel and/or the datatable invisible and make the histogram
% axes visible
SelectHistogramDataPanel(hObject, eventdata, handles, 'Histogram')

% Find the multiplying factor
expon = floor(log10(max(results.DelaunayAreaBinEdges)));
DelaunayAreaBinEdges = results.DelaunayAreaBinEdges/10^expon;

% Plot the bins as filled 2D polygons
freq = results.DelaunayAreaBinCount/sum(results.DelaunayAreaBinCount);
for i = 1:length(DelaunayAreaBinEdges)-1
    hB(i) = fill([DelaunayAreaBinEdges(i) DelaunayAreaBinEdges(i)...
        DelaunayAreaBinEdges(i+1) DelaunayAreaBinEdges(i+1)],...
        [0 freq(i) freq(i) 0],'b');
    hold on
end
hold off
set(gca,'xlim',[min(DelaunayAreaBinEdges),max(DelaunayAreaBinEdges)],...
    'ButtonDownFcn','SpatialAnalysis(''ButtonDownPlotAxes'',gcbo,[],guidata(gcbo))',...
    'DeleteFcn','SpatialAnalysis(''DeleteHistogramAxes'',gcbo,[],guidata(gcbf))')
xlabel(['Surface area (x10^',num2str(expon),' ',options.Units,'^2)'])
ylabel('Frequency (%)')
title('Delaunay Facet Area')

% Calculate the average, standard deviation, min and max of the population
NNavg = mean(results.DelaunayArea(results.DelaunayAreaHistIndex))/10^expon;
NNstd = std(results.DelaunayArea(results.DelaunayAreaHistIndex))/10^expon;
minNN = min(results.DelaunayArea(results.DelaunayAreaHistIndex))/10^expon;
maxNN = max(results.DelaunayArea(results.DelaunayAreaHistIndex))/10^expon;

% Print these values on the histogram
hB(i+1) = text(.98,.99,...
    {['Avg=',num2str(NNavg,'%4.2f'),', \sigma=',num2str(NNstd,'%4.2f'),...
    ', min=',num2str(minNN,'%4.2f')];['max=',num2str(maxNN,'%4.2f'),...
    ' (x10^',num2str(expon),' ',options.Units,'^2)']},...
    'VerticalAlignment','top','HorizontalAlignment','right',...
    'EdgeColor',[0 0 0],'units','normalized');

% Save the plot data
PlotData.handles = hB;
PlotData.type = 'Histogram';
PlotData.style = 'DelaunayArea'; 

% --------------------------------------------------------------------
function VoronoiVolumeHistogram_Callback(hObject, eventdata, handles)
% This function will plot the voronoi diagram statistical data in a
% histogram form
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Need to check if the points have been plotted, if not, plot them
if(~strcmp(PlotData.type,'PlotPoints'))
    PlotPoints(hObject, eventdata, handles,0)
    colorbar off
end

% Get the Voronoi Indices appropriate for the filter
A = GetFilteredVoronoiIndices(hObject, eventdata, handles, results);
    
% Get all the voronoi volumes, calculate the bin edges, then
% perform the histogram calculation.
results.VoronoiVolBinEdges = ...
    [0:options.VoronoiVolBinSize:options.VoronoiVolBinSize*options.NumVoronoiVolBins,inf];

% Check to make sure we have points to do the calculation
if(isempty(A))
    % Clear out some variables
    results.VoronoiVolBinCount = [];
    results.VoronoiVolBins = [];
    delete(get(handles.HistogramAxes,'children'));
    PlotData.handles = [];
    
    % Create a warning dialog box
    msg = ['No Voronoi Domains are available under the current filter: ',...
        options.VoronoiFilter];
    hW = warndlg(msg,'3D Spatial Analysis-Delaunay Calculation','modal');
    uiwait(hW)
    SpatialAnalysisMessage(msg)
    
    % Save the data
    data{1} = results;
    data{3} = PlotData;
    set(gcbf,'UserData',data)
    return
else
    % Perform the histogram calculation
    [results.VoronoiVolBinCount,results.VoronoiVolBins] =...
        histc(results.VoronoiVolume(A),results.VoronoiVolBinEdges);
    results.VoronoiVolBinEdges(length(results.VoronoiVolBinEdges)) = ...
        results.VoronoiVolBinEdges(length(results.VoronoiVolBinEdges)-1)+...
        options.VoronoiVolBinSize;
end

% Make the datatable invisible and make the histogram axes visible
SelectHistogramDataPanel(hObject, eventdata, handles, 'Histogram')

% Find the multiplying factor
expon = floor(log10(max(results.VoronoiVolBinEdges)));
VoronoiVolBinEdges = results.VoronoiVolBinEdges/10^expon;

% Plot the bins as filled 2D polygons
freq = results.VoronoiVolBinCount/sum(results.VoronoiVolBinCount);
for i = 1:length(VoronoiVolBinEdges)-1
    hB(i) = fill([VoronoiVolBinEdges(i) VoronoiVolBinEdges(i)...
        VoronoiVolBinEdges(i+1) VoronoiVolBinEdges(i+1)],...
        [0 freq(i) freq(i) 0],'b');
    hold on
end
hold off
set(gca,'xlim',[min(VoronoiVolBinEdges),max(VoronoiVolBinEdges)],...
    'XScale','linear','ButtonDownFcn',...
    'SpatialAnalysis(''ButtonDownPlotAxes'',gcbo,[],guidata(gcbo))',...
    'DeleteFcn','SpatialAnalysis(''DeleteHistogramAxes'',gcbo,[],guidata(gcbf))')
xlabel(['Volume (x10^',num2str(expon),' ',options.Units,'^3)'])
ylabel('Frequency (%)')
title('Voronoi Volume')

% Calculate the average, standard deviation, min and max of the population
NNavg = mean(results.VoronoiVolume(A))/10^expon;
NNstd = std(results.VoronoiVolume(A))/10^expon;
minNN = min(results.VoronoiVolume(A))/10^expon;
maxNN = max(results.VoronoiVolume(A))/10^expon;

% Print these values on the histogram
hB(i+1) = text(.98,.99,...
    {['Avg=',num2str(NNavg,'%4.2f'),', \sigma=',num2str(NNstd,'%4.2f'),...
    ', min=',num2str(minNN,'%4.2f')];['max=',num2str(maxNN,'%4.2f'),...
    ' (x10^',num2str(expon),' ',options.Units,'^3)']},...
    'VerticalAlignment','top','HorizontalAlignment','right',...
    'EdgeColor',[0 0 0],'units','normalized');

% Save the plot data information
PlotData.handles = hB;
PlotData.type = 'Histogram';
PlotData.style = 'VoronoiVolume';

% Save all the data
data{1} = results;
data{3} = PlotData;
set(gcbf,'UserData',data)

%--------------------------------------------------------------------------
function results = VoronoiHistogramCalc(hObject, eventdata, handles, results)
% This function will calculate all the voronoi histogram statistics
data = get(gcf,'UserData');
options = data{2};

% Get the Voronoi Indices appropriate for the filter
A = GetFilteredVoronoiIndices(hObject, eventdata, handles, results);

% Get all the voronoi areas, calculate the bin edges, then
% perform the histogram calculation.
results.VoronoiAreaBinEdges = ...
    [0:options.VoronoiAreaBinSize:options.VoronoiAreaBinSize*options.NumVoronoiAreaBins,inf];
[results.VoronoiAreaBinCount,results.VoronoiAreaBins] =...
    histc(results.VoronoiArea(A),results.VoronoiAreaBinEdges);
results.VoronoiAreaBinEdges(length(results.VoronoiAreaBinEdges)) = ...
    results.VoronoiAreaBinEdges(length(results.VoronoiAreaBinEdges)-1)+...
    options.VoronoiAreaBinSize;

% Get all the voronoi volumes, calculate the bin edges, then
% perform the histogram calculation.
results.VoronoiVolBinEdges = ...
    [0:options.VoronoiVolBinSize:options.VoronoiVolBinSize*options.NumVoronoiVolBins,inf];

% Perform the histogram calculation
[results.VoronoiVolBinCount,results.VoronoiVolBins] =...
    histc(results.VoronoiVolume(A),results.VoronoiVolBinEdges);
results.VoronoiVolBinEdges(length(results.VoronoiVolBinEdges)) = ...
    results.VoronoiVolBinEdges(length(results.VoronoiVolBinEdges)-1)+...
    options.VoronoiVolBinSize;


% --------------------------------------------------------------------
function VoronoiDirVectorsMenu_Callback(hObject, eventdata, handles)
% This function will plot the direction vectors for the voronoi volumes
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Get the Voronoi Indices appropriate for the filter
A = GetFilteredVoronoiIndices(hObject, eventdata, handles, results);

% Check to see if we have any points to plot
if(isempty(A))
    SpatialAnalysisMessage('No points found for the given Voronoi filter');
    return
end

% Select the Plot Axes and delete any previous items
SelectAxes(hObject, eventdata, handles, 'PlotAxes')
FixAxisLimits(hObject, eventdata, handles, 'PlotAxes')

B = find(ishandle(PlotData.handles));
if(any(B))
    delete(PlotData.handles(B))
    delete(get(gca,'children'))
else
    delete(get(gca,'children'))
end 


% Round to the nearest 10th so that vectors that are very close to zero but
% not EXACTLY zero, do not look like vectors
VoronoiDirVectors = round(results.VoronoiDirVectors*10)/10;

% Plot the direction vectors as arrows (vectors)
h = quiver3(results.X(A),results.Y(A),results.Z(A),VoronoiDirVectors(A,1),...
        VoronoiDirVectors(A,2),VoronoiDirVectors(A,3),'b');
PrintAxisLabels(hObject, eventdata, handles)
title('Voronoi Domain Direction Vectors')
box on
FixAxisLimits(hObject, eventdata, handles,'PlotAxes')

% Save the plot data;
PlotData.handles = h;
PlotData.type = 'VoronoiDirVectors';
PlotData.style = 'VoronoiDirVectors';

% Perform a histogram on the data
x = [-1:.1:1];
if(length(A)==1)
    for i = 1:3
        count(:,i) = hist(results.VoronoiDirVectors(A,i),x);
    end
else
    count = hist(results.VoronoiDirVectors(A,:),x);
end

% Make the datatable invisible and make the histogram axes visible
SelectHistogramDataPanel(hObject, eventdata, handles, 'Histogram')

% Plot the histogram as a bar chart
bar(x,count./length(A),'stacked')
legend('x','y','z','location','best')
xlabel('Normalized Direction Vector')
ylabel('Frequency')
title('Voronoi Domain Direction Vectors')
set(gca,'xlim',[-1.1 1.1]);

% Save all the data
data{1} = results;
data{3} = PlotData;
set(gcbf,'UserData',data)

%-------------------------------------------------------------------------
function UpdateCurVoronoiDirVect(hObject, eventdata, handles)
return

data = get(gcf,'UserData');
results = data{1};

% Get the Voronoi Indices appropriate for the filter
A = GetFilteredVoronoiIndices(hObject, eventdata, handles, results);
a = results.CurPoint;

if(sum(A==a))
    VoronoiDirVectors = round(results.VoronoiDirVectors(a,:)*10)/10;
    h = quiver3('v6',results.X(a),results.Y(a),results.Z(a),VoronoiDirVectors(1),...
        VoronoiDirVectors(2),VoronoiDirVectors(3),'r');
    %set(h,'linewidth',4)
end

% --------------------------------------------------------------------
function VoronoiAreaHistogram_Callback(hObject, eventdata, handles)
% This function will plot the voronoi diagram statistical data in a
% histogram form
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Need to check if the points have been plotted, if not, plot them
if(~strcmp(PlotData.type,'PlotPoints'))
    PlotPoints(hObject, eventdata, handles,0)
    colorbar off
end

% Get the Voronoi Indices appropriate for the filter
A = GetFilteredVoronoiIndices(hObject, eventdata, handles, results);

% Get all the voronoi areas, calculate the bin edges, then
% perform the histogram calculation.
results.VoronoiAreaBinEdges = ...
    [0:options.VoronoiAreaBinSize:options.VoronoiAreaBinSize*options.NumVoronoiAreaBins,inf];

% Check to make sure we have points to do the calculation
if(isempty(A))
    % Clear out some variables
    results.VoronoiAreaBinCount = [];
    results.VoronoiAreaBins = [];
    delete(get(handles.HistogramAxes,'children'));
    PlotData.handles = [];
    
    % Create a warning dialog box
    msg = ['No Voronoi Domains are available under the current filter: ',...
        options.VoronoiFilter];
    hW = warndlg(msg,'3D Spatial Analysis-Delaunay Calculation','modal');
    uiwait(hW)
    SpatialAnalysisMessage(msg)
    
    % Save the data
    data{1} = results;
    data{3} = PlotData;
    set(gcbf,'UserData',data)
    return
else
    
[results.VoronoiAreaBinCount,results.VoronoiAreaBins] =...
    histc(results.VoronoiArea(A),results.VoronoiAreaBinEdges);
results.VoronoiAreaBinEdges(length(results.VoronoiAreaBinEdges)) = ...
    results.VoronoiAreaBinEdges(length(results.VoronoiAreaBinEdges)-1)+...
    options.VoronoiAreaBinSize;
end

% Make the datapanel and/or the datatable invisible and make the histogram
% axes visible
SelectHistogramDataPanel(hObject, eventdata, handles, 'Histogram')

% Find the multiplying factor
expon = floor(log10(max(results.VoronoiAreaBinEdges)));
VoronoiAreaBinEdges = results.VoronoiAreaBinEdges/10^expon;

% Plot the bins as filled 2D polygons
freq = results.VoronoiAreaBinCount/sum(results.VoronoiAreaBinCount);
for i = 1:length(VoronoiAreaBinEdges)-1
    hB(i) = fill([VoronoiAreaBinEdges(i) VoronoiAreaBinEdges(i)...
        VoronoiAreaBinEdges(i+1) VoronoiAreaBinEdges(i+1)],...
        [0 freq(i) freq(i) 0],'b');
    hold on
end
hold off
set(gca,'xlim',[min(VoronoiAreaBinEdges),max(VoronoiAreaBinEdges)],...
    'ButtonDownFcn','SpatialAnalysis(''ButtonDownPlotAxes'',gcbo,[],guidata(gcbo))',...
    'DeleteFcn','SpatialAnalysis(''DeleteHistogramAxes'',gcbo,[],guidata(gcbf))')
xlabel(['Surface area (x10^',num2str(expon),' ',options.Units,'^2)'])
ylabel('Frequency (%)')
title('Voronoi Surface Area')

% Calculate the average, standard deviation, min and max of the population
NNavg = mean(results.VoronoiArea(A))/10^expon;
NNstd = std(results.VoronoiArea(A))/10^expon;
minNN = min(results.VoronoiArea(A))/10^expon;
maxNN = max(results.VoronoiArea(A))/10^expon;

% Print these values on the histogram
hB(i+1) = text(.98,.99,...
    {['Avg=',num2str(NNavg,'%4.2f'),', \sigma=',num2str(NNstd,'%4.2f'),...
    ', min=',num2str(minNN,'%4.2f')];['max=',num2str(maxNN,'%4.2f'),...
    ' (x10^',num2str(expon),' ',options.Units,'^2)']},...
    'VerticalAlignment','top','HorizontalAlignment','right',...
    'EdgeColor',[0 0 0],'units','normalized');

% Save the plot data
PlotData.handles = hB;
PlotData.type = 'Histogram';
PlotData.style = 'VoronoiSurfaceArea';

% Save all the data
data{1} = results;
data{3} = PlotData;
set(gcbf,'UserData',data)

% --------------------------------------------------------------------
function AutocorrelationHistogram_Callback(hObject, eventdata, handles)
% This function will plot the autocorrelation histogram
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Select the Autocorelation axes
SelectAxes(hObject, eventdata, handles, 'AutocorrelationAxes')

% Make the the datatable invisible and make the histogram axes visible
SelectHistogramDataPanel(hObject, eventdata, handles, 'Histogram')

% Plot the histogram bars
ACbins = results.ACBins;
freq = results.ACCount./results.ACBinVolume;
warning off
expon = floor(max(log10(freq)));
freq = freq/10^expon;
warning on

% Plot the autocorrelation data as a histogram
for i = 1:options.NumACBins
    hB(i) = fill([ACbins(i) ACbins(i) ACbins(i+1) ACbins(i+1)],...
        [0 freq(i) freq(i) 0],'b');
    hold on
end

% Plot the horizontal line that represents the density of interest and then
% the vertical line that represents the effective radius.
ACDensity = results.ACDensity*numel(results.ACIndices);
hB(i+1) = plot([min(ACbins),max(ACbins)],ACDensity/10^expon*[1 1],'-k');
hB(i+2) = plot(results.ACreff*[1 1],[0 ACDensity/10^expon],'-k');

hold off
set(gca,'xlim',[min(ACbins),max(ACbins)],'ButtonDownFcn',...
    'SpatialAnalysis(''ButtonDownPlotAxes'',gcbo,[],guidata(gcbo))',...
    'DeleteFcn','SpatialAnalysis(''DeleteHistogramAxes'',gcbo,[],guidata(gcbf))')
xlabel(['Distance (',options.Units,')'])
ylabel(['Count/Volume (x10^{',num2str(expon),'} #/',options.Units,'^3)'])
title('Density Recovery Profile')

% Print the effective radius on the plot in a text box (upper left)
hB(i+3) = text(.02,.99,['r_{eff} = ',num2str(results.ACreff,'%4.2f'),...
    options.Units],...
    'VerticalAlignment','top','HorizontalAlignment','left',...
    'EdgeColor',[0 0 0],'units','normalized');

% Save the plot data
PlotData.handles = hB;
PlotData.type = 'Histogram';

% Save all the data
data{1} = results;
data{3} = PlotData;
set(gcbf,'UserData',data)


% --------------------------------------------------------------------
function SpatialAnalysisFig_KeyPressFcn(hObject, eventdata, handles)
% This function will handle any key presses
data = get(gcf,'UserData');
results = data{1};
PlotData = data{3};

% Get the current character hit
CurrentCharacter = get(gcf,'CurrentCharacter');
if(isempty(CurrentCharacter))
    return
end

% We want to perform different tasks for the key presses depending on which
% analysis we are looking at
switch(PlotData.type)
    case 'Delaunay'
        % Remember the previous tetrahedron
        PreviousPoint = results.CurTetra;
        switch(CurrentCharacter*1)
            case 51
                % The user pressed the 3 key
                % Call the 3D perspective view
                ThreeDPerspectiveMenu_Callback(hObject, eventdata, handles)
            case 27 
                % The escape key
                set(handles.STOPButton,'UserData',1)
            case 28
                % left is 28
                results.CurTetra = max(results.CurTetra-10,1);
            case 29
                % right is 29
                results.CurTetra = min(results.CurTetra+10,size(results.T,1));
            case 30
                %up is 30
                results.CurTetra = min(results.CurTetra+1,size(results.T,1));
            case 31
                %down is 31
                results.CurTetra = max(results.CurTetra-1,1);

            case 's'
                return
                % THIS FUNCTION MAKES NO SENSE AT THIS TIME
                
                % Increases the delaunay section number

                % See if we have performed a near neighbor analysis
                if(~results.NNCALC)
                    return
                end

                % Get the number of Delaunay Facets
                NumDelaunayFacets = size(results.DelaunayK{results.CurPoint},1);

                % Increase the current section by one, but only go up to the
                % maximum number of facets, then go back to 1 (mod)
                results.CurSection(results.CurPoint) = ...
                    mod(results.CurSection(results.CurPoint)+1,NumDelaunayFacets)+1;

                % Save the data here
                data{1} = results;
                set(gcf,'UserData',data)

                % Plot the delaunay section of interest
                PlotDelaunaySection(hObject, eventdata, handles)
                return
        end

        % If we are on the same tetrahedron, there's no need to do anything
        if(PreviousPoint==results.CurTetra)
            return
        end
    otherwise
        % Remember the previous point
        PreviousPoint = results.CurPoint;
        switch(CurrentCharacter*1)
            case 51
                % The user pressed the 3 key
                % Call the 3D perspective view
                ThreeDPerspectiveMenu_Callback(hObject, eventdata, handles)
            case 27
                % The escape key
                
            case 28
                % left is 28
                results.CurPoint = max(results.CurPoint-10,1);
            case 29
                % right is 29
                results.CurPoint = min(results.CurPoint+10,results.NumPoints);
            case 30
                %up is 30
                results.CurPoint = min(results.CurPoint+1,results.NumPoints);
            case 31
                %down is 31
                results.CurPoint = max(results.CurPoint-1,1);
        end

        % If we are on the same point, there's no need to do anything
        if(PreviousPoint==results.CurPoint)
            return
        end

end

        
% Save the results
data{1} = results;
set(gcf,'UserData',data)

% Update the figure based on the current point
UpdateOnCurrentPoint(hObject, eventdata, handles, PreviousPoint)

%--------------------------------------------------------------------------
function UpdateOnCurrentPoint(hObject, eventdata, handles, PreviousPoint)
% This function will update the current plot based on the current point and
% current plot type
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Plot with the new current point
switch(PlotData.type)
    case 'PlotPoints'
        delete(findobj('tag','CurPoint'))
        SelectAxes(hObject, eventdata, handles,'PlotAxes')
        plot3(results.X(results.CurPoint),results.Y(results.CurPoint),...
            results.Z(results.CurPoint),'*r','tag','CurPoint',...
            'markersize',options.MarkerSize)
        UpdateDataTable(hObject, eventdata, handles)
    case 'Delaunay'
        DelaunayMenu_Callback(hObject, eventdata, handles)
    case 'Voronoi'
        VoronoiDiagramsMenu_Callback(hObject, eventdata, handles)
    case 'NearestNeighbor'
        NearestNeighborMenu_Callback(hObject, eventdata, handles)
    case 'FalseColor'
        UpdateDataTable(hObject, eventdata, handles)
        set(PlotData.handles(PreviousPoint),'markersize',...
            get(PlotData.handles(PreviousPoint),'UserData'))
        set(PlotData.handles(results.CurPoint),'markersize',15)
    case 'VoronoiDirVectors'
        UpdateCurVoronoiDirVect(hObject, eventdata, handles)
    otherwise
        %UpdateDataTable(hObject, eventdata, handles)
end

%--------------------------------------------------------------------------
function SelectPoint(hObject, eventdata, handles)
% This function will select the point that the user clicked on
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Remember the previous point
PreviousPoint = results.CurPoint;

% Get the point that was selected
Points = get(gca,'CurrentPoint');
p1 = Points(1,:);
p2 = Points(2,:);

% Get the distance of the hypotenuse (distance from point 1 to the data
% point)
h = sqrt((results.X-p1(1)).^2 + (results.Y-p1(2)).^2 + ...
    (results.Z-p1(3)).^2);

% Calclate the magnitude of the cross product between the vectors that are
% formed between the data point and the two cursor points
for i = 1:size(results.points,1)
    p3 = results.points(i,:);
    sinalpha(i,1) = norm(cross((p2-p1)/norm(p2-p1),(p3-p1)/norm(p3-p1)));
end

[mind,results.CurPoint] = min(h.*sinalpha);

% If we are on the same point, there's no need to do anything
if(PreviousPoint==results.CurPoint)
    return
end

% Save the results
data{1} = results;
set(gcf,'UserData',data)

% Update the figure based on the current point
UpdateOnCurrentPoint(hObject, eventdata, handles, PreviousPoint)

% --------------------------------------------------------------------
function ZoomInMenu_Callback(hObject, eventdata, handles)
% Zooms in the plot by 2X
if(strcmp(get(handles.PlotAxes,'visible'),'on'))
    SelectAxes(hObject, eventdata, handles, 'PlotAxes')
else
    SelectAxes(hObject, eventdata, handles, 'AutocorrelationAxes')
end
zoom(2)


% --------------------------------------------------------------------
function ZoomOutMenu_Callback(hObject, eventdata, handles)
% Zooms out of the plot by 1/2 FOV
if(strcmp(get(handles.PlotAxes,'visible'),'on'))
    SelectAxes(hObject, eventdata, handles, 'PlotAxes')
else
    SelectAxes(hObject, eventdata, handles, 'AutocorrelationAxes')
end
zoom(1/2)

% --------------------------------------------------------------------
function PanMenu_Callback(hObject, eventdata, handles)
% hObject    handle to PanMenu (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
function Rotate3DMenu_Callback(hObject, eventdata, handles)
% Turns on the manual rotation
if(strcmp(get(handles.PlotAxes,'visible'),'on'))
    SelectAxes(hObject, eventdata, handles, 'PlotAxes')
else
    SelectAxes(hObject, eventdata, handles, 'AutocorrelationAxes')
end
rotate3d

% --------------------------------------------------------------------
function AutoRotateMenu_Callback(hObject, eventdata, handles)
% Automatically rotate the plot
if(strcmp(get(handles.PlotAxes,'visible'),'on'))
    SelectAxes(hObject, eventdata, handles, 'PlotAxes')
else
    SelectAxes(hObject, eventdata, handles, 'AutocorrelationAxes')
end

% Fix the title's location
%itleloc = get(get(gca,'title'));

% See if we have a colorbar
hCB = findobj('Tag','Colorbar');
if(~isempty(hCB))
    oldloc = get(hCB,'location');
    oldpos = get(gca,'position');
    set(hCB,'location','manual')
    set(gca,'position',oldpos)
end
% Rotate the axis by -5 degrees every 100 ms
rotate3d off
view(3)
axis vis3d
EL = 30;

for AZ = -37.5:-5:-397.5
    view(AZ,EL)
    pause(.1)
end


% Set the colorbar location back to it's default setting
if(~isempty(hCB))
    set(hCB,'location',oldloc)
    set(gca,'position',oldpos)
end

% --------------------------------------------------------------------
function EqualAspectRatioMenu_Callback(hObject, eventdata, handles)
% This function will set the aspect ratio to 1:1:1 so that a sphere looks
% like a sphere
data = get(gcf,'UserData');
options = data{2};

if(strcmp(get(handles.PlotAxes,'visible'),'on'))
    SelectAxes(hObject, eventdata, handles, 'PlotAxes')
    % Set the axis limits correctly
    FixAxisLimits(hObject, eventdata, handles, 'PlotAxes')
else
    SelectAxes(hObject, eventdata, handles, 'AutocorrelationAxes')
    % Set the axis limits correctly
    FixAxisLimits(hObject, eventdata, handles, 'AutocorrelationAxes')
end
axis equal


% Set the checkmarks on the menus
set(handles.EqualAspectRatioMenu,'checked','on')
set(handles.AutoAspectRatioMenu,'checked','off')

% Save in the options the type of aspect ratio
options.AspectRatio = 'Equal';
data{2} = options;
set(gcf,'UserData',data)

% --------------------------------------------------------------------
function AutoAspectRatioMenu_Callback(hObject, eventdata, handles)
% This function will set the axis scaling to its default, automatic
% mode where, for each dimension, 'nice' limits are chosen based
% on the extents of all line, surface, patch, and image children.
data = get(gcf,'UserData');
options = data{2};

if(strcmp(get(handles.PlotAxes,'visible'),'on'))
    SelectAxes(hObject, eventdata, handles, 'PlotAxes')
else
    SelectAxes(hObject, eventdata, handles, 'AutocorrelationAxes')
end
axis square

% Set the checkmarks on the menus
set(handles.EqualAspectRatioMenu,'checked','off')
set(handles.AutoAspectRatioMenu,'checked','on')

% Save in the options the type of aspect ratio
options.AspectRatio = 'Auto';
data{2} = options;
set(gcf,'UserData',data)

% --------------------------------------------------------------------
function XYPlaneMenu_Callback(hObject, eventdata, handles)
% Puts the axis in the x-y plane
if(strcmp(get(handles.PlotAxes,'visible'),'on'))
    SelectAxes(hObject, eventdata, handles, 'PlotAxes')
else
    SelectAxes(hObject, eventdata, handles, 'AutocorrelationAxes')
end
view(0,90)


% --------------------------------------------------------------------
function XZPlaneMenu_Callback(hObject, eventdata, handles)
% Puts the axis in the x-z plane
if(strcmp(get(handles.PlotAxes,'visible'),'on'))
    SelectAxes(hObject, eventdata, handles, 'PlotAxes')
else
    SelectAxes(hObject, eventdata, handles, 'AutocorrelationAxes')
end
view(0,0)

% --------------------------------------------------------------------
function YZPlaneMenu_Callback(hObject, eventdata, handles)
% Puts the axis in the y-z plane
if(strcmp(get(handles.PlotAxes,'visible'),'on'))
    SelectAxes(hObject, eventdata, handles, 'PlotAxes')
else
    SelectAxes(hObject, eventdata, handles, 'AutocorrelationAxes')
end
view(90,0)

% --------------------------------------------------------------------
function ThreeDPerspectiveMenu_Callback(hObject, eventdata, handles)
% Set the view to the default 3D view
if(strcmp(get(handles.PlotAxes,'visible'),'on'))
    SelectAxes(hObject, eventdata, handles, 'PlotAxes')
else
    SelectAxes(hObject, eventdata, handles, 'AutocorrelationAxes')
end
zoom out
view(3)

% --------------------------------------------------------------------
function SetViewMenu_Callback(hObject, eventdata, handles)
% Manually set the view/orientation of the plot
if(strcmp(get(handles.PlotAxes,'visible'),'on'))
    SelectAxes(hObject, eventdata, handles, 'PlotAxes')
else
    SelectAxes(hObject, eventdata, handles, 'AutocorrelationAxes')
end
viewdlg(gca)

% --------------------------------------------------------------------
function NewRandomSetMenu_Callback(hObject, eventdata, handles)
% Create a new set of random data
ERROR = NewRandomSet(hObject, eventdata, handles);

% Plot the fake default points.
if(~ERROR)
    PlotPoints(hObject, eventdata, handles)
end

% --------------------------------------------------------------------
function OptionsMenu_Callback(hObject, eventdata, handles)
% This function will call the options window and then compare to see if any
% calculation flags need to be reset.
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Call the SAOptions function and pass the current options.  Then, wait for
% the window to change it's visible tag
hSAO = SAOptions(options);
waitfor(hSAO,'visible')
if(ishandle(hSAO))
    % Need to get the new options structure
    newoptions = get(hSAO,'UserData');
    close(hSAO)
else
    % The user canceled the options menu
    return
end

% Set the options equal to the new options.  Then, check to see which items
% if any have changed so that we can reset the calculation flags
SAME = cmpstruct(options,newoptions);
if(~SAME)
    if(options.NumACBins ~= newoptions.NumACBins | ...
            options.MaxRadiusAC ~= newoptions.MaxRadiusAC | ...
            ~strcmp(options.EdgeCorrectionAC,newoptions.EdgeCorrectionAC) | ...
            ~strcmp(options.FilterAC,newoptions.FilterAC) | ...
            options.PLOTAC ~= newoptions.PLOTAC | ...
            options.MarkerSizeAC ~= newoptions.MarkerSizeAC)
        results.ACCALC = 0;
    end

    % Check to see if the units have changed
    if(~strcmp(options.Units,newoptions.Units))
        PrintAxisLabels(hObject, eventdata, handles, newoptions.Units)
        
        % Check to see if a false color plot is active and if so, update
        % the title of the colorbar
        if(strncmp(PlotData.type,'False',5))
            % Turn the colorbar on and set the tick labels
            FixColorbarAxisLabels(hObject, eventdata, handles, newoptions.Units)
        end
    end
    
    % Check to see if the boundaries have changed.  If so, we need to
    % recalculate everything because the filtering will change.
    if(options.MaxX ~= newoptions.MaxX | options.MaxY ~= newoptions.MaxY | ...
            options.MaxZ ~= newoptions.MaxZ)
        results.ACCALC = 0;
        results.NNCALC = 0;
        results.VORONOICALC = 0;
        
        % Now check to see if the user made the boundaries too small.
        if(newoptions.MaxX < max(results.X) | newoptions.MaxY < max(results.Y) | ...
                newoptions.MaxZ < max(results.Z))
            Answer = questdlg({'One or more of the boundaries has been set smaller than the data.';...
                'Would you like to reset the boundaries?'},'Spatial Analysis 3D','Yes','No','Yes');
            switch(Answer)
                case 'Yes'
                    newoptions.MaxX = options.MaxX;
                    newoptions.MaxY = options.MaxY;
                    newoptions.MaxZ = options.MaxZ;
            end
        end 
    end
    
    % Recalculate the sample volume in case
    results.SampleVolume = (newoptions.MaxX-newoptions.MinX)*...
        (newoptions.MaxY-newoptions.MinY)*(newoptions.MaxZ-newoptions.MinZ);
end

% Save the data
data{1} = results;
data{2} = newoptions;
set(gcf,'UserData',data)

% Update the summary table
UpdateSummaryTable(hObject, eventdata, handles)

% --------------------------------------------------------------------
function SpatialAnalysisFig_WindowButtonMotionFcn(hObject, eventdata, handles)
% Set the time in the window
set(handles.TimeTXT,'string',datestr(now,16))


% -------------------------------------------------------------------------
function MSGBox_Callback(hObject, eventdata, handles)
% No callback

% -------------------------------------------------------------------------
function MSGBox_CreateFcn(hObject, eventdata, handles)
% Set the message box background color to that of the figure
set(hObject,'BackgroundColor',get(gcf,'color'));

% --------------------------------------------------------------------
function SpatialAnalysisMessage(msg1)
% Writes a message to the message box
handles = guihandles(findobj('Tag','SpatialAnalysisFig'));
msg = get(handles.MSGBox,'UserData');
LEN = length(msg);
msg{LEN+1} = msg1;

set(handles.MSGBox,'string',msg,'UserData',msg)
drawnow

% --------------------------------------------------------------------
function SpatialAnalysisMessageDONE
% Writes the word DONE in the previous line of the message box
handles = guihandles(findobj('Tag','SpatialAnalysisFig'));
msg = get(handles.MSGBox,'UserData');
LEN = length(msg);
msg{LEN} = [msg{LEN},'DONE'];

set(handles.MSGBox,'string',msg,'UserData',msg)


%--------------------------------------------------------------------------
function results = ClearResults(hObject, eventdata, handles, varargin)
% This function will clear the results structure.  Think of it as a
% constructor function

% The user should have passed the results structure.  See if the user also
% passed the number of points that should be initialized
len = 1;
dir = pwd;
if(length(varargin)==1)
    results = varargin{1};
    dir = results.DataPath;
elseif(length(varargin)==2)
    results = varargin{1};
    if(isempty(results))
        dir = pwd;
    else
        dir = results.DataPath;
    end
    len = varargin{2};
end

EmptyAry = NaN*ones(len,1);

results.filename = [];                              % Current filename
results.X = [];                                     % X coordinates
results.Y = [];                                     % Y coordinates
results.Z = [];                                     % Z coordinates
results.Area = EmptyAry;                            % measured area
results.Volume = EmptyAry;                          % measured volume
results.Eccentricity = EmptyAry;                    % Eccentricity
results.Circularity = EmptyAry;                     % Circularity
results.Sphericity = EmptyAry;                      % Sphericity
results.K = [];                                     % cell array Voronoi Vertices
results.DataPath = dir;                             % current data path
results.NumPoints = len;                            % Number of data points
results.CurPoint = 1;                               % the current data point
results.CurSection = ones(len,1);                   % Current slice for the delaunay diagram
results.CurTetra = 1;                               % Current tetrahedron being rendered
results.points = EmptyAry;                          % [x,y,z] data point coordinates
results.NNCALC = 0;                                 % Nearest Neighbor Calculation Flag
results.VORONOICALC = 0;                            % Voronoi Calculation Flag
results.ACCALC = 0;                                 % Autocorrelation Flag
results.ConvexHullIndices = [];                     % Convex Hull Indices
results.NumVoronoiVertices = EmptyAry;              % Number of Voronoi Vertices
results.NumVoronoiFacets = EmptyAry;                % Number of Voronoi Facets
results.NumNN = EmptyAry;                           % Number of Nearest Neighbors
results.MinNNDist = EmptyAry;                       % Minimum near neighbor distance
results.MaxNNDist = EmptyAry;                       % Maximum near neighbor distance 
results.AvgNNDist = EmptyAry;                       % Average near neighbor distance
results.NNIndices = [];                             % Near neighbor indices
results.NNIndex = [];                               % NearEST neighbor indices
results.NNClass = EmptyAry;                         % NN point class
results.NNDistHist = [];                            % NN min dist for histogram
results.NNBinEdges = [];                            % NN Bin edges for the histogram
results.NNBinCount = [];                            % NN Bin count
results.NNBins = [];                                % NN Bin location for each point
results.DelaunaySegLen = [];                        % Delaunay segment lengths
results.DelaunayArea = [];                          % Delaunay surface areas
results.DelaunayTotalArea = [];                     % Delaunay total surface area
results.DelaunayVolume = [];                        % Delaunay volumes
results.DelaunayMinArea = EmptyAry;
results.DelaunayMaxArea = EmptyAry;
results.DelaunayMinVol = EmptyAry;
results.DelaunayMaxVol = EmptyAry;
results.SegLenIndex = [];                           % Indices of all the Delaunay segments
results.FacetIndex = [];                            % Indices of all the Delaunay facets
results.DelaunayNumFacets = EmptyAry;
results.DelaunayVolHistIndex = [];                  % Filtered Indices of the Delaunay Volumes
results.DelaunayVolBinEdges = [];
results.DelaunayVolBinCount = [];
results.DelaunayVolBins = [];
results.DelaunayAreaHistIndex = [];                 % Filtered Indices of the Delaunay Facet Areas
results.DelaunayAreaBinEdges = [];
results.DelaunayAreaBinCount = [];
results.DelaunayAreaBins = [];
results.DelaunaySegLenHistIndex = [];               % Delaunay seg len indices used in the histogram
results.DelaunaySegLenBinEdges = [];                % Delaunay seg len bin edges for hist
results.DelaunaySegLenBinCount = [];                % Delaunay seg len bin count
results.DelaunaySegLenBins = [];                    % Delaunay seg len bin location for each point
results.T = [];                                     % Delaunay tesselation
results.V = [];                                     % Voronoi verticies
results.C = [];                                     % Voronoi cells
results.VoronoiArea = EmptyAry;                     % Voronoi surface areas
results.VoronoiAreaBinEdges = [];                   % Voronoi area histogram bin edges
results.VoronoiAreaBinCount = [];                   % Voronoi area histogram count
results.VoronoiAreaBins = [];                       % Voronoi area point -> bin
results.VoronoiVolume = EmptyAry;                   % Voronoi volumes
results.VoronoiVolBinEdges = [];                    % Voronoi Volume histogram bin edges
results.VoronoiVolBinCount = [];                    % Voronoi Volume histogram bin count
results.VoronoiVolBins = [];                        % Voronoi volume point -> bin
results.VoronoiClass = EmptyAry;                    % Voronoi point class
results.Comment = [];                               % Comment about the point
results.ACBins = [];                                
results.ACCount = [];                               % AC histogram count
results.ACIndices = [];
results.ACBinVolume = [];                           % Bin volume for the AC
results.SampleVolume = [];                          % Total sample space volume
results.ACDensity = [];                             % Point density in the AC
results.ACDc = [];                                  % AC Critical density
results.ACreff = [];                                % AC Effective Radius
results.ACrm = [];
results.ACk = [];                                   % AC reliability factor
results.ACp = [];                                   % AC packing factor
results.ACx = [];                                   % x-pos of the points in the AC
results.ACy = [];                                   % y-pos of the points in the AC
results.ACz = [];                                   % z-pos of the points in the AC
results.VoronoiDirVectors = zeros(len,3);
results.kr = [];                                    % K-function radius
results.ky = [];                                    % K-function data
results.kpoisson = [];                              % K-function with poisson distr
results.gr = [];                                    % G-function radius
results.gy = [];                                    % G-function data
results.gpoisson = [];                              % G-function with poisson distr
results.fr = [];                                    % F-function radius
results.fy = [];                                    % F-function data
results.fpoisson = [];                              % F-function with poisson distr

% These variables will be cell arrays, so we need to empty each cell array.
% They are all related to Voronoi analysis
for a = 1:len
    results.V{a} = [];              
    results.C{a} = [];
    results.Comment{a} = [];
    results.K{a} = [];                  % cell array Voronoi Vertices
end


%--------------------------------------------------------------------------
function hP = InitializeSpatialAnalysisWaitBar(hObject, eventdata, handles, varargin)
% Make the WaitBar Axes the current one and create a patch

axes(handles.SpatialAnalysisWaitBar)
hold on
hP = patch([0 0 .00 .00],[0 1 1 0],'b');
set(hP,'Tag','SpatialAnalysisWaitBar')
set(gca,'xlim',[0 1])

% Make sure the plot axes is current
axes(handles.PlotAxes)

%--------------------------------------------------------------------------
function SpatialAnalysisWaitBar(x)
% This function will increment the waitbar

if(x>1 | x<0)
    return
end

% Set the Xdata for the waitbar
handles = guidata(findobj('tag','SpatialAnalysisFig'));
Xdata = get(handles.WaitBar,'Xdata');
Xdata(3:4) = x;
set(handles.WaitBar,'XData',Xdata)
drawnow

%--------------------------------------------------------------------------
function UpdateSummaryTable(hObject, eventdata, handles)
% This function will update the first table, the summary table
data = get(gcf,'UserData');
results = data{1};
options = data{2};

% Update each value in the data table
Table = get(handles.DataTable1Title,'UserData');
set(Table(3,2),'string',results.NumPoints)
set(Table(4,2),'string',numel(results.ConvexHullIndices))
set(Table(5,2),'string',results.SampleVolume)

%--------------------------------------------------------------------------
function UpdateDataTable(hObject, eventdata, handles)
% This function will update the data table with the current values
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Turn warnings off
warning off

% Update each value in the data table.  In the first table, we need to
% update the current point number and the current point (x,y,z)
if(results.CurPoint==0)
    return
end
set(handles.DataTable1R1C2,'string',num2str(results.CurPoint,'%d'));
set(handles.DataTable1R2C2,'string',['(',...
    num2str(results.points(results.CurPoint,:),'%6.2f,%6.2f,%6.2f'),')'])
set(handles.DataTable1R2C3,'string',options.Units)
set(handles.DataTable1R5C3,'string',options.Units)
set(handles.DataTable1Subscript5,'string','3')

% Now get the handles for the entire data table
Table = get(handles.DataTable2Title,'UserData');

switch(get(handles.DataTable2Title,'value'))
    case 1
        % General
        set(Table(1,2),'string',num2str(results.Area(results.CurPoint),...
            options.Precision));
        set(Table(2,2),'string',num2str(results.Volume(results.CurPoint),...
            options.Precision));
        set(Table(3,2),'string',num2str(results.Eccentricity(results.CurPoint),...
            options.Precision));
        set(Table(4,2),'string',num2str(results.Circularity(results.CurPoint),...
            options.Precision));
        set(Table(5,2),'string',num2str(results.Sphericity(results.CurPoint),...
            options.Precision));
        
        % Now set the units
        set(Table(1,3),'string',options.Units)
        set(Table(1,4),'string','2')
        set(Table(2,3),'string',options.Units)
        set(Table(2,4),'string','3')
        
    case 2
        % Near Neighbor Analysis
        %Metrics = {'Near Neighbors';'Avg. Distance';'Min Distance';...
        %    'Max Distance';'Near Neighbor Class'};
        set(Table(1,2),'string',results.NumNN(results.CurPoint))
        set(Table(2,2),'string',results.AvgNNDist(results.CurPoint))
        set(Table(3,2),'string',results.MinNNDist(results.CurPoint))
        set(Table(4,2),'string',results.MaxNNDist(results.CurPoint))
        set(Table(5,2),'string',results.NNClass(results.CurPoint))
        set(Table(6,2),'string',options.NNFilter)

        % Now set the units
        set(Table(2:4,3),'string',options.Units)
        set(Table(2:4,4),'string','')
    
    case 3
        % Delaunay Analysis   
%         Metrics = {'# of Tetrahedra';'Tetrahedron Number';...
%             'Tetrahedron Points';'Tetrahedron Volume';...
%             'Tetrahedron Surf Area';'Point Classification';'Current Filter'};    

          set(handles.DataTable1R1C2,'string','N/A');
          set(handles.DataTable1R2C2,'string','N/A');
          if(isempty(results.T))
              return
          end
          set(Table(1,2),'string',size(results.T,1));
          set(Table(2,2),'string',results.CurTetra);
          set(Table(3,2),'string',num2str(results.T(results.CurTetra,:)));
          set(Table(4,2),'string',num2str(results.DelaunayVolume(results.CurTetra,:),'%8.2e'));
          set(Table(5,2),'string',num2str(results.DelaunayTotalArea(results.CurTetra,:),'%8.2e'));
          set(Table(6,2),'string',num2str(results.NNClass(results.T(results.CurTetra,:))'))
          set(Table(7,2),'string',options.NNFilter)

          % Now set the units
          set(Table(4:5,3),'string',options.Units)
          set(Table(4,4),'string','3')
          set(Table(5,4),'string','2')
        
    case 4
        % Voronoi Analysis
        %Metrics = {'Voronoi Vertices'; 'Voronoi Facets';...
        %    'Voronoi Surface Area';'Voronoi Volume';'Voronoi Class'};
        set(Table(1,2),'string',results.NumVoronoiVertices(results.CurPoint))
        set(Table(2,2),'string',results.NumVoronoiFacets(results.CurPoint))
        set(Table(3,2),'string',num2str(sum(results.VoronoiArea(results.CurPoint)),...
            options.Precision));
        set(Table(4,2),'string',num2str(results.VoronoiVolume(results.CurPoint),...
            options.Precision));
        VoronoiDirVector = round(results.VoronoiDirVectors(results.CurPoint,:)*100)/100;
        set(Table(5,2),'string',['<',...
            num2str(VoronoiDirVector(1)),',',...
            num2str(VoronoiDirVector(2)),',',...
            num2str(VoronoiDirVector(3)),'>'])
        set(Table(6,2),'string',results.VoronoiClass(results.CurPoint))
        set(Table(7,2),'string',options.VoronoiFilter)
        
        % Now set the units
        set(Table(3:4,3),'string',options.Units)
        set(Table(3,4),'string','2')
        set(Table(4,4),'string','3')
    case 5
        % Autocorrelation
        %Metrics = {'Density';'Critical Density';'Effective Radius';...
        %   'rm';'Reliability Factor';'Packing Factor'};
        set(Table(1,2),'string',num2str(results.ACDensity,'%0.3e'));
        set(Table(2,2),'string',num2str(results.ACDc,'%0.3e'));
        set(Table(3,2),'string',num2str(results.ACreff,options.Precision));
        set(Table(4,2),'string',num2str(results.ACrm,options.Precision));
        set(Table(5,2),'string',num2str(results.ACk,options.Precision));
        set(Table(6,2),'string',num2str(results.ACp,options.Precision));
        set(Table(7,2),'string',options.FilterAC);
        set(Table(8,2),'string',options.EdgeCorrectionAC);

        % Now set the units
        set(Table(1:4,3),'string',options.Units)
        set(Table(1:2,4),'string','-3')
end

% Turn warnings back on
warning on

% Make the datapanel and/or the datatable visible and make the histogram
% axes invisible
SelectHistogramDataPanel(hObject, eventdata, handles, 'DataPanel')


% -------------------------------------------------------------------------
function ClearDataTable(hObject, eventdata, handles)
% Clear the values in the data table
Table1Handles = get(handles.DataTable1Title,'UserData');
for i = 1:length(Table1Handles)
    set(Table1Handles(i,2),'string','')
end

Table2Handles = get(handles.DataTable2Title,'UserData');
for i = 1:length(Table2Handles)
    set(Table2Handles(i,2),'string','')
end


% -------------------------------------------------------------------------
function PlotAxes_ButtonDownFcn(hObject, eventdata, handles)
%
get(gca,'CurrentPoint');


% --------------------------------------------------------------------
function AboutSpatialAnalysis_Callback(hObject, eventdata, handles)
% hObject    handle to AboutSpatialAnalysis (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
AboutSpatialAnalysis

% --------------------------------------------------------------------
function TipOfTheDayMenu_Callback(hObject, eventdata, handles)
% This function will bring up the Tip of the Day Dialog
TipOfTheDay

% --------------------------------------------------------------------
function FindDataExtent(hObject, eventdata, handles)
% Determine the extent of the data
data = get(gcf,'UserData');
results = data{1};
options = data{2};

% Determine the extent of the data
xmin = floor(min(results.points(:,1)));
xmax = ceil(max(results.points(:,1)));
ymin = floor(min(results.points(:,2)));
ymax = ceil(max(results.points(:,2)));
zmin = floor(min(results.points(:,3)));
zmax = ceil(max(results.points(:,3)));

% The user may have already set the data boundaries in the options.  So,
% ask the user if they would like to keep the current limits or reset them
% to the data

answer = questdlg('Would you like to set the boundaries to the data?',...
    'Boundary Definition','Yes Reset Boundaries',...
    'No Keep Current Boundaries','Yes Reset Boundaries');

switch(answer)
    case 'Yes Reset Boundaries'
        %options.MinX = xmin;
        
        options.MaxX = xmax;
        %options.MinY = ymin;
        
        options.MaxY = ymax;
        %options.MinZ = zmin;
        
        options.MaxZ = zmax;
    otherwise
        % Do nothing
end

results.SampleVolume = (options.MaxX-options.MinX)*...
    (options.MaxY-options.MinY)*(options.MaxZ-options.MinZ);

% Save the data
data{1} = results;
data{2} = options;
set(gcf,'UserData',data)

% Fix the axis limits
FixAxisLimits(hObject, eventdata, handles, 'PlotAxes')

% -------------------------------------------------------------------------
function txt = FormatDataText(hObject, eventdata, handles, datasum)
numchar = 50;

txt1 = char(datasum(:,1));
txt3 = char(datasum(:,2));
txt2 = char(32*ones(length(datasum),5));
txt = [txt1,txt2,txt3];

% -------------------------------------------------------------------------
function UpdateRecentFiles(hObject, eventdata, handles, varargin)
%

% Get all the recently opened files they are stored in the preferences
for i = 1:4
    file{i} = getpref('RecentFiles',['File',num2str(i)]);
end

% If the user passed a filename, then we may need to add the file to the
% preferences
if(length(varargin))
    newfile = varargin{1};
    cmp = strcmp(file,newfile);
    if(~any(cmp))
        for i = 4:-1:2
            file{i} = file{i-1};
        end
        file{1} = newfile;
    else
        A = find(cmp);
        tempfile = file{A};
        for i = A:-1:2
            file{i} = file{i-1};
        end
        file{1} = newfile;
    end
end


% Set all the recently opened files they are stored in the preferences
for i = 1:4
    % First see if the filename is empty.  If it is not, then create the
    % label that will go into the menu.  Need to create the callback too
    hF = findobj('Tag',['RecentFile',num2str(i)]);
    if(~strcmp(file{i},''))
        A = findstr(file{i},filesep);
        if(numel(A)>1)
            filename = file{i};
            filestr = [num2str(i),' ',filename(1:A(1)),'...',...
                filename(A(length(A)):length(filename))];
        else
            filestr = [num2str(i),' ',file{i}];
        end
        set(hF,'label',filestr,'visible','on','callback',...
            'SpatialAnalysis(''OpenMenu_Callback'',gcbo,[],guidata(gcbf),1)',...
            'UserData',filename)
    else
        set(hF,'label','','visible','off','callback','',...
            'UserData',[])
    end
    setpref('RecentFiles',['File',num2str(i)],file{i});
end

% -------------------------------------------------------------------------
function SelectAxes(hObject, eventdata, handles, selectaxes)
% Select the correct axis based on the axes that was passed to the function

switch(selectaxes)
    case 'PlotAxes'
        axes(handles.AutocorrelationAxes)
        colorbar off
        axes(handles.PlotAxes)
        set(handles.PlotAxes,'visible','on')
        set(get(handles.PlotAxes,'children'),'visible','on')
        set([handles.AutocorrelationAxes;...
            get(handles.AutocorrelationAxes,'children')],'visible','off')
    case 'AutocorrelationAxes'
        axes(handles.PlotAxes)
        colorbar off
        legend off
        axes(handles.AutocorrelationAxes)
        set([handles.PlotAxes;...
            get(handles.PlotAxes,'children')],'visible','off')
        set(handles.AutocorrelationAxes,'visible','on')
        set(get(handles.AutocorrelationAxes,'children'),'visible','on')
        hCB = colorbar;
        hT = title(hCB,get(gca,'UserData'));
end

%--------------------------------------------------------------------------
function SelectHistogramDataPanel(hObject, eventdata, handles, AxesPanel)
% Choose either the histogram axes or the data panel
switch(AxesPanel)
    
    case 'Histogram'
        % Select the histogram axes, and hide the data panel
        set([handles.DataTable1Title,handles.DataTable2Title],...
            'visible','off')
        set([get(handles.DataTable1Title,'UserData');...
            get(handles.DataTable2Title,'UserData')],'visible','off')
        
        % Make the histogram axis visible and current
        set(handles.HistogramAxes,'visible','on')
        set(get(handles.HistogramAxes,'children'),'visible','on')
        axes(handles.HistogramAxes)
    case 'DataPanel'
        % Set the histogram axes off and make the data panel visible
        axes(handles.HistogramAxes)
        legend off
        set(handles.HistogramAxes,'visible','off')
        set(get(handles.HistogramAxes,'children'),'visible','off')
        set([handles.DataTable1Title,handles.DataTable2Title],...
            'visible','on')
        set([get(handles.DataTable1Title,'UserData');...
            get(handles.DataTable2Title,'UserData')],'visible','on')
end


% -------------------------------------------------------------------------
function ERROR = NewRandomSet(hObject, eventdata, handles)
data = get(gcf,'UserData');
results = data{1};
options = data{2};
ERROR = 0;

% Check to see if the volume around each point (4/3*PI*dmin^3)*NumRandPoints 
% > the sample volume space
RandSetVolume = options.NumRandPoints*4/3*pi*options.RandomAvgDist^3;
XW = options.MaxX-options.MinX;
YW = options.MaxY-options.MinY;
ZW = options.MaxZ-options.MinZ;
SampleVolume = XW*YW*ZW;        
MaxIter = 100;        
MaxNumPts = 5000;

if(RandSetVolume>SampleVolume)
    answer = questdlg({'The required sample space for the simulated data extends';...
        'past the set boundaries';'Do you wish to continue?'},'New Simulated Data Set',...
        'Yes Reset Boundaries','No Keep Current Boundaries','Yes Reset Boundaries');
    switch(answer)
        case 'Yes Reset Boundaries'
            options.MinX = 0;
            options.MinY = 0;
            options.MinZ = 0;
            RandSetRadius = (RandSetVolume*3/4/pi)^(1/3);
            options.MaxX = ceil(RandSetRadius);
            options.MaxY = options.MaxX;
            options.MaxZ = options.MaxX;
            XW = options.MaxX-options.MinX;
            YW = options.MaxY-options.MinY;
            ZW = options.MaxZ-options.MinZ;
            SampleVolume = XW*YW*ZW;

        case {'No','Cancel'}
            ERROR = 1;
            return
    end
end

switch(options.SimulatedDataType)
    % Generate the simulated data based on the type of simulated data
    case 'HCP'
        % Hexagonal Close Packed
        %options.NumRandPoints
        d = options.RandomAvgDist;
        vol = 4/3*pi*d^3;
        NumPoints = floor(SampleVolume/vol);

        if(NumPoints>MaxNumPts)
            answer = questdlg({...
                ['The number of HCP points required to fill the sample space (',...
                num2str(NumPoints),')'];['is greater than ',num2str(MaxNumPts),...
                '  Do you wish to continue?']},'New Simulated Data Set',...
                'Yes continue','No do NOT continue','No do NOT continue');
            switch(answer)
                case 'No do NOT continue'
                    return
                otherwise
                    % Do nothing
            end
        end

        z = 0;
        ODDZ = true;
        pt = 1;
        while(z<=ZW)
            
            % Make one layer of cells.
            % Here is the y offset for the different layers.
            if(ODDZ)
                y = 0;
                ISODD = true;
            else
                y = (d/( 2*sqrt(3)));
                ISODD = false;
            end
            
            while( y <= YW)
                if(ISODD)
                    x = 0;
                else
                    x = d/2;
                end

                % put in one row of cells.
                while (x <= XW)
                    p(pt,:) = [x,y,z]; 
                    x = x + d; 
                    pt = pt + 1;
                end

                % end of a row, get ready for next row:
                y = y + (d * (sqrt(3)/2));
                ISODD = ~ISODD;
            end

            z = z + (d * sqrt(2/3));
            ODDZ = ~ODDZ;
        end
 
        % We need to add "noise" or variation to the HCP data
        p = p+(randn(size(p))*options.RandomSTD);
        
        % Tell the user we are done
        SpatialAnalysisMessageDONE
        
    case 'Random'
        % This function will generate a random set based on average
        % distance and standard deviation
        SpatialAnalysisWaitBar(0);
        SpatialAnalysisMessage('Generating New Random Set...')
        set(gcbf,'Name','3D Spatial Analysis - Random Set');

        % If we have dmin3d as a mex file, then just call that file
        if(exist('dmin3d')==3)
            [p,r,d,i] = dmin3d(options.NumRandPoints, [XW YW ZW],...
                [options.RandomAvgDist, options.RandomSTD, ...
                options.RandomMinDist, -1]);

            % check to see if the simulation failed (e.g. too many points
            % for the given spacing
            if(p(1,1)==-1)
                SpatialAnalysisMessage('dmin3d algorithm failed.  Try fewer points or closer spacing')
                return
            end
            
            % Tell the user we are done
            SpatialAnalysisMessageDONE
            
            % Calculate the packing factor, or fill factor
            pf = sum(4/3*pi*(d/2).^3)/(XW*YW*ZW);
            
            % Tell the user how many rejects there were
            SpatialAnalysisMessage(['Number of rejects: ',num2str(i(1)),])
            SpatialAnalysisMessage(['Packing density: ',num2str(i(2)*100),'%'])
            
            
            
        else

            % Random data set

            % Generate the first random point
            p = rand(1,3).*[XW YW ZW] + [options.MinX options.MinY options.MinZ];
            ITER = 1;

            for  i = 1:options.NumRandPoints-1
                % generate newpoint within the volume
                newpoint = rand(1,3).*[XW YW ZW] + ...
                    [options.MinX options.MinY options.MinZ];

                % Generate a minimal distance value from normal distribution with
                % mean of RandomAvgDist and stdev RandomSTD. dmin also has
                % to be greater than or equal to RandomMinDist
                dmin = -1;
                while(dmin<options.RandomMinDist)
                    dmin = (randn(1,1)*options.RandomSTD) + options.RandomAvgDist;
                end

                % Check to see if the new point is within dmin and that we are
                % less than the maximum number of tries
                while(any(((p(:,1)-newpoint(1)).^2 + (p(:,2)-newpoint(2)).^2 + ...
                        (p(:,3)-newpoint(3)).^2)<= dmin.^2) & ITER<MaxIter)

                    % Do not need to check any more for the point being within
                    %    the volume, as we know it must be the way it is created.

                    newpoint = rand(1,3).*[XW YW ZW] + ...
                        [options.MinX options.MinY options.MinZ];

                    % Generate a new dmin is optional, but I think it best.
                    dmin = (randn(1,1)*options.RandomSTD) + options.RandomAvgDist;
                    ITER = ITER+1;
                end
                p = [p;newpoint];
            end

            % Tell the user we are done
            SpatialAnalysisMessageDONE
        end
end


%  Reset the waitbar
SpatialAnalysisWaitBar(0);
nodata = NaN*ones(options.NumRandPoints,1);

% Clear the results structure
[NumPoints,A] = size(p);
results = ClearResults(hObject, eventdata, handles,[],NumPoints);
results.points = p;
results.X = p(:,1);
results.Y = p(:,2);
results.Z = p(:,3);
results.NumPoints = NumPoints;
results.Area = nodata;
results.Volume = nodata;
results.Eccentricity = nodata;
results.Circularity = nodata;
results.Sphericity = nodata;
results.SampleVolume = SampleVolume;

% Find the indicies that correspond to the convex hull
K = convhulln(results.points);
K = unique(K);

% Save the indices of the convex hull
results.ConvexHullIndices = K;

data{1} = results;
data{2} = options;
set(gcf,'UserData',data);

% Set the Summary to General
set(handles.DataTable2Title,'value',1)
DataTable2Title_Callback(hObject, eventdata, handles)

% Update the Summary Table
UpdateSummaryTable(hObject, eventdata, handles)

% Enable/disable the false color menu items
EnableFalseColorMenues(hObject, eventdata, handles)

%--------------------------------------------------------------------------
function InitializeDataTable(hObject, eventdata, handles)
% This function will initialize the data table by creating a set of handles
% for the table itself and storing those handles as a matrix that is then
% stored inside the data table titles
for i = 1:7
    Table1(i,:) = [getfield(handles,['DataTable1R',num2str(i),'C1']),...
        getfield(handles,['DataTable1R',num2str(i),'C2']),...
        getfield(handles,['DataTable1R',num2str(i),'C3']),...
        getfield(handles,['DataTable1Subscript',num2str(i)])];
end
for i = 1:10
    Table2(i,:) = [getfield(handles,['DataTable2R',num2str(i),'C1']),...
        getfield(handles,['DataTable2R',num2str(i),'C2']),...
        getfield(handles,['DataTable2R',num2str(i),'C3']),...
        getfield(handles,['DataTable2Subscript',num2str(i)])];
end

set(handles.DataTable1Title,'UserData',Table1)
set(handles.DataTable2Title,'UserData',Table2)


%--------------------------------------------------------------------------
function DataTable1R1C2_Callback(hObject, eventdata, handles)
% This function will select the point that the user clicked on
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Remember the previous point
PreviousPoint = results.CurPoint;

Point = str2num(get(hObject,'string'));
if(isempty(Point))
    set(hObject,'string',PreviousPoint)
    return
elseif(Point<1 | Point>results.NumPoints)
    set(hObject,'string',PreviousPoint)
    return
else
    results.CurPoint = Point;
end

% If we are on the same point, there's no need to do anything
if(PreviousPoint==results.CurPoint)
    return
end

% Save the results
data{1} = results;
set(gcf,'UserData',data)

% Update the figure based on the current point
UpdateOnCurrentPoint(hObject, eventdata, handles, PreviousPoint)


%--------------------------------------------------------------------------
function DataTable2Title_Callback(hObject, eventdata, handles)
% This function is called when the Data Table Title is changed
data = get(gcf,'UserData');
PlotData = data{3};

% Get the handles for all the cells in the table
Table2Handles = get(handles.DataTable2Title,'UserData');

switch(get(handles.DataTable2Title,'value'))
    case 1
        % General        
        Metrics = {'Area';'Volume';'Eccentricity';'Circularity';...
            'Sphericity'};
        if(~strcmp(PlotData.type,'PlotPoints'))
            PlotPoints(hObject, eventdata, handles)
        end
    case 2
        % Near Neighbor Analysis
        if(hObject==handles.DataTable2Title)
            NearestNeighborMenu_Callback(hObject, eventdata, handles)
        end
        Metrics = {'Near Neighbors';'Avg. Distance';'Min Distance';...
            'Max Distance';'Point Classification';'Current Filter'};
    case 3
        % Delaunay Analysis
        if(hObject==handles.DataTable2Title)
            DelaunayMenu_Callback(hObject, eventdata, handles)
        end
        Metrics = {'# of Tetrahedra';'Tetrahedron Number';...
            'Tetrahedron Points';'Tetrahedron Volume';...
            'Tetrahedron Surf Area';'Point Classification';'Current Filter'};    
    case 4
        % Voronoi Domains
        if(hObject==handles.DataTable2Title)
            VoronoiDiagramsMenu_Callback(hObject, eventdata, handles)
        end
        Metrics = {'Voronoi Vertices'; 'Voronoi Facets';...
            'Voronoi Surface Area';'Voronoi Volume';...
            'Direction Vector';'Point Classification';'Current Filter'};
    case 5
        % Autocorrelation
        if(hObject==handles.DataTable2Title)
            AutocorrelationMenu_Callback(hObject, eventdata, handles);
        end
        Metrics = {'Density';'Critical Density';'Effective Radius';...
            'Max Radius';'Reliability Factor';'Packing Factor';...
            'Current Filter';'Edge Correction'};
end

% Set the metric titles and clear the values in the entire table
for i = 1:numel(Metrics)
    set(Table2Handles(i,1),'string',Metrics{i})
    set(Table2Handles(i,2:4),'string','')
end
for i = numel(Metrics)+1:length(Table2Handles)
    set(Table2Handles(i,1:4),'string','')
end

% Update data table
UpdateDataTable(hObject, eventdata, handles)


%--------------------------------------------------------------------------
function PrintAxisLabels(hObject, eventdata, handles, varargin)
% This function will print the axis labels with the correct units into the
% plot figure on the left.  It will also fix the aspect ratio
data = get(gcf,'UserData');
options = data{2};
PlotData = data{3};

% Check to see if the user passed the units to use in the labels
if(length(varargin))
    Units = varargin{1};
else
    data = get(gcf,'UserData');
    options = data{2};
    Units = options.Units;
end

% Set the x,y, and z labels
xlabel(['x (',Units,')'])
ylabel(['y (',Units,')'])
zlabel(['z (',Units,')'])

% Based on the value of the Aspect ratio flag, call the right function
switch(options.AspectRatio)
    case 'Auto'
        AutoAspectRatioMenu_Callback(hObject, eventdata, handles)
    case 'Equal'
        EqualAspectRatioMenu_Callback(hObject, eventdata, handles)
end

%-------------------------------------------------------------------------
function FixAxisLimits(hObject, eventdata, handles, selectaxes)
data = get(gcf,'UserData');
options = data{2};

% Select the correct axis and then set the limits from the options
switch(selectaxes)
    case 'PlotAxes'
        set(gca,'xlim',[options.MinX options.MaxX],...
            'ylim',[options.MinY options.MaxY],...
            'zlim',[options.MinZ options.MaxZ])
    case 'AutocorrelationAxes'
        set(gca,'xlim',[-options.MaxRadiusAC options.MaxRadiusAC],...
            'ylim',[-options.MaxRadiusAC options.MaxRadiusAC],...
            'zlim',[-options.MaxRadiusAC options.MaxRadiusAC])
end

%--------------------------------------------------------------------------
function FixColorbarAxisLabels(hObject, eventdata, handles, varargin)
% This function will add the tick labels and title to the colorbar
data = get(gcf,'UserData');
options = data{2};

% Check to see if the user passed the desired units
if(length(varargin))
    Units = varargin{1};
else
    data = get(gcf,'UserData');
    options = data{2};
    Units = options.Units;
end

% Find out what order of magnitude the data is then divide by that
% magnitude
cmax = get(gca,'UserData');
expon = floor(log10(cmax));
cmax = cmax/10^expon;

% Create a colorbar and determine the tick labels
hCB = colorbar;
dy = cmax/10;
ytick = [0:dy:cmax];
yticklabel = num2str(ytick',2);
set(hCB,'ytick',ytick/cmax*100,'yticklabel',...
    yticklabel,'ylim',[0 100]);

% If our exponent was 0 then we just write the title otherwise we add a
% x10^expon to the title
dim = 3;

switch(get(hObject,'label'))
    case {'Circularity','Eccentricity','Sphericity'}
        Units = 'A.U.';
        dim = ' ';
    case {'Nearest Neighbor Distance','Avg Delaunay Segment Length'}
        dim = 1;
    case {'Delaunay Areas','Voronoi Surface Areas','Point Areas'}
        dim = 2;        
    case 'Options'
        dim = getappdata(gca,'dimension');
        if(strcmp(dim,' '))
            Units = 'A.U.';
        end
    otherwise
        dim = 3;
end

% Remember the dimension of the plot
setappdata(gca,'dimension',dim)

% If we don't have an exponent then we shouldn't use the ^ in the text
if(expon)
    if(dim==1)
        hT = title(hCB,['(x10^',num2str(expon),' ',Units,')']);        
    else
        hT = title(hCB,['(x10^',num2str(expon),' ',Units,'^',num2str(dim),')']);
    end
else
    if(dim==1)
        hT = title(hCB,['(',Units,')'],'Tag','ColorbarTitle');
    else
        hT = title(hCB,['(',Units,'^',num2str(dim),')'],'Tag','ColorbarTitle');
    end
end

% Remember the title's handle inside the colorbar's userdata
set(hCB,'UserData',hT)

% --------------------------------------------------------------------
function RestoreMarkerSizeMenu_Callback(hObject, eventdata, handles)
% This function will remove the large marker size that may have been
% created within the plot so that the user could recognize the current
% point 
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Plot with the new current point
switch(PlotData.type)
    case 'FalseColor'
        set(PlotData.handles(results.CurPoint),'markersize',...
            get(PlotData.handles(results.CurPoint),'UserData'))
    otherwise
        %Do nothing
end

%--------------------------------------------------------------------------
function EnableFalseColorMenues(hObject, eventdata, handles)
% This function when called, will check to see if data exists in the proper
% location for plotting various false color plots
data = get(gcf,'UserData');
results = data{1};

if(all(isnan(results.Area)))
    set(handles.FalseColorAreaMenu,'enable','off')
else
    set(handles.FalseColorAreaMenu,'enable','on')
end
if(all(isnan(results.Volume)))
    set(handles.FalseColorVolumeMenu,'enable','off')
else
    set(handles.FalseColorVolumeMenu,'enable','on')
end
if(all(isnan(results.Eccentricity)))
    set(handles.FalseColorEccentricityMenu,'enable','off')
else
    set(handles.FalseColorEccentricityMenu,'enable','on')
end
if(all(isnan(results.Circularity)))
    set(handles.FalseColorCircularityMenu,'enable','off')
else
    set(handles.FalseColorCircularityMenu,'enable','on')
end
if(all(isnan(results.Sphericity)))
    set(handles.FalseColorSphericityMenu,'enable','off')
else
    set(handles.FalseColorSphericityMenu,'enable','on')
end


% --------------------------------------------------------------------
function ViewDataTableMenu_Callback(hObject, eventdata, handles)
% Make sure the data in the table is accurate
UpdateDataTable(hObject, eventdata, handles)

%This function switches from the histogram view to the data panel view
SelectHistogramDataPanel(hObject, eventdata, handles, 'DataPanel')


% --------------------------------------------------------------------
function ClearMessageWindowMenu_Callback(hObject, eventdata, handles)
% This function will clear the message window
msg = get(handles.MSGBox,'UserData');
msg = [];

set(handles.MSGBox,'string',msg,'UserData',msg)
drawnow

% --------------------------------------------------------------------
function ButtonDownPlotAxes(hObject, eventdata, handles)
% This function is called when the plot axis is click on

switch(get(gcf,'SelectionType'))
    case 'open'
        propedit(gca)
end



% --------------------------------------------------------------------
function KFunctionMenu_Callback(hObject, eventdata, handles)
% This function will call the k-function mex file and calculate the
% cummulative DRP

% Make sure the data table panel is showing the general tab
set(handles.DataTable2Title,'value',1)
DataTable2Title_Callback(hObject, eventdata, handles)

data = get(gcf,'Userdata');
results = data{1};
options = data{2};

% Call the K-Function calculation function
results = KFunctionCalc(hObject, eventdata, handles, results);

data{1} = results;
set(gcf,'UserData',data)

% Plot the cummulative Density Recovery Profile
SelectAxes(hObject, eventdata, handles, 'PlotAxes')
hold off
plot(results.kr,results.ky,results.kr,results.kpoisson,'--')
xlabel(['Distance (',options.Units,')'])
ylabel(['K(d) (',options.Units,'^3)'])
title('Cumulative Density Recovery Profile')
legend('K','Poisson','location','NorthWest')
grid on
set(gca,'ButtonDownFcn',...
    'SpatialAnalysis(''ButtonDownPlotAxes'',gcbo,[],guidata(gcbo))')

%--------------------------------------------------------------------------
function results = KFunctionCalc(hObject, eventdata, handles, results);
% The k3d function is called with the following:
%
%   PTS     array of size [N,3] where N is number of points.
%   BOX     vector of length 6 giving [XLO XHI YLO YHI ZLO ZHI]
%   RANGE   vector of length 3 [LO HI NVAL] saying the distances
%           at which K should be evaluated.
%   METHOD  integer: 0 (translation) or 1 (isotropic)% Return values:
% 
%   Return values:
%   matrix with 2 columns and NVAL rows, giving the distance r, and
%   the value of the K function at that distance r.
data = get(gcf,'Userdata');
options = data{2}; 

range = [0 options.KFcnRange options.KFcnNumPoints];
box = [0 options.MaxX 0 options.MaxY 0 options.MaxZ];
METHOD = 0;

% Call the k3d function which calls the mex function
res = k3d(results.points, box, range, METHOD);

% Save the results
results.kr = res(:,1);
results.ky = res(:,2);
results.kpoisson = 4/3*pi*(res(:,1).^3);

% --------------------------------------------------------------------
function varargout = FFunctionMenu_Callback(hObject, eventdata, handles)
% This function will perform the F-function 

% Make sure the data table panel is showing the general tab
set(handles.DataTable2Title,'value',1)
DataTable2Title_Callback(hObject, eventdata, handles)

data = get(gcf,'Userdata');
results = data{1};
options = data{2};

% Call the F-Function calculation function
results = FFunctionCalc(hObject, eventdata, handles, results);

data{1} = results;
set(gcf,'UserData',data)

% Plot the cummulative Density Recovery Profile
SelectAxes(hObject, eventdata, handles, 'PlotAxes')
hold off
plot(results.fr,results.fy,results.fr,results.fpoisson,'--')
legend('F','Poisson','location','SouthEast')
xlabel(['Distance (',options.Units,')'])
title('Cumulative Sample Point to Nearest Cell Distances')
ylabel('F(d)')
grid on
set(gca,'ButtonDownFcn',...
    'SpatialAnalysis(''ButtonDownPlotAxes'',gcbo,[],guidata(gcbo))')

%--------------------------------------------------------------------------
function results = FFunctionCalc(hObject, eventdata, handles, results)
%   F3D - Compute F function in 3d.
%   This is a wrapper around kfg_funs, but first we do some error checking.
%  
%   PTS     array of size [N,3] where N is number of points.
%   BOX     vector of length 6 giving [XLO XHI YLO YHI ZLO ZHI]
%   RANGE   vector of length 3 [LO HI NVAL] saying the distances
%           at which K should be evaluated.
%   VSIZE   size of a voxel (See notes).
%   METHOD  integer: 
%           0 (no correction)
%           1 (minus sampling).
data = get(gcf,'Userdata');
options = data{2};

range = [0 options.FFcnRange options.FFcnNumPoints];
box = [0 options.MaxX 0 options.MaxY 0 options.MaxZ];
METHOD = 0;
vside = max([options.MaxX,options.MaxY,options.MaxZ])/options.fdivisions;

% Call the f3d function which calls the mex function
res = f3d(results.points, box, range, vside, METHOD);

% Save the results
results.fr = res(:,1);
results.fy = res(:,2);
intensity = size(results.points,1)/((box(2)-box(1))*(box(4)-box(3))*(box(6)-box(5)));
results.fpoisson = 1 - exp( -4/3*intensity*pi*results.fr.^3);

% --------------------------------------------------------------------
function GFunction_Callback(hObject, eventdata, handles)
% This function will perform the G-function

% Make sure the data table panel is showing the general tab
set(handles.DataTable2Title,'value',1)
DataTable2Title_Callback(hObject, eventdata, handles)

data = get(gcf,'Userdata');
results = data{1};
options = data{2};

% Call the G-Function Calculation function
results = GFunctionCalc(hObject, eventdata, handles, results);

% Plot the cummulative Density Recovery Profile
SelectAxes(hObject, eventdata, handles, 'PlotAxes')
hold off
plot(results.gr,results.gy,results.gr,results.gpoisson,'--')
legend('G','Poisson','location','SouthEast')
xlabel(['Distance (',options.Units,')'])
title('Cumulative Nearest-Neighbor Distribution')
ylabel('G(d)')
grid on
set(gca,'ButtonDownFcn',...
    'SpatialAnalysis(''ButtonDownPlotAxes'',gcbo,[],guidata(gcbo))')

%--------------------------------------------------------------------------
function results = GFunctionCalc(hObject, eventdata, handles, results)
%   G3D - Compute G function in 3d.
%   This is a wrapper around kfg_funs, but first we do some error checking.
%  
%   PTS     array of size [N,3] where N is number of points.
%   BOX     vector of length 6 giving [XLO XHI YLO YHI ZLO ZHI]
%   RANGE   vector of length 3 [LO HI NVAL] saying the distances
%           at which G should be evaluated.
%   METHOD  integer: 
%           1 (minus sampling).
%           3 (Hanisch G3).
data = get(gcf,'Userdata');
options = data{2};

range = [0 options.GFcnRange options.GFcnNumPoints];
box = [0 options.MaxX 0 options.MaxY 0 options.MaxZ];
METHOD = 0;

% Call the g3d function which calls the mex function
res = g3d(results.points, box, range, METHOD);

% Save the results
results.gr = res(:,1);
results.gy = res(:,2);
intensity = size(results.points,1)/((box(2)-box(1))*(box(4)-box(3))*(box(6)-box(5)));
results.gpoisson = 1 - exp( -4/3*intensity*pi*results.gr.^3);
data{1} = results;
set(gcf,'UserData',data)

% --------------------------------------------------------------------
function ACTriHistogramMenu_Callback(hObject, eventdata, handles)
% This function will plot the Autocorrelation tri histogram plot
data = get(gcf,'Userdata');
results = data{1};
options = data{2};

% Convert the x,y,z data from the autocorrelation into spherical
% coordinates.
[theta,phi,rho] = cart2sph(results.ACx,results.ACy,results.ACz);

figure
% Set the position to be the middle of the screen
screensize = get(0,'MonitorPositions');
pos = get(gcf,'position');
% Double the width
pos(3) = pos(3)*2;
pos(1) = max((screensize(3)-pos(3))/2,0);
pos(2) = max((screensize(4)-pos(4))/2,0);
set(gcf,'position',pos);

% Plot the radial statistics
subplot(1,3,1)
N = histc(rho,results.ACBins);
Nrho = N(1:end-1);
Nrho(end) = Nrho(end)+N(end);
ACbins = results.ACBins;
freq = results.ACCount./results.ACBinVolume;
bar(ACbins(1:end-1),freq./sum(freq)*100,'histc')
xlabel(['Radial distance (',options.Units,')'])
ylabel('Normalized count/volume (%)')
title('Radius')

% Calculate the statistics for the angle theta
xtheta = [-pi:2*pi/(options.ACThetaBins-1):pi]';
Ntheta = histc(theta,xtheta);

% The last bin contains all points that exactly equal pi (which is the same
% as -pi.  So, take the last bin count and add half to the first bin and
% half to the one before the last bin
Ntheta(length(Ntheta)-1) = Ntheta(length(Ntheta)-1)+Ntheta(length(Ntheta))/2;
Ntheta(1) = Ntheta(1) + Ntheta(length(Ntheta))/2;

% Normalize the count to a frequency.  Theta does not have a scaling or
% normalizing factor like radius or phi. The theta slices are all the same
% "volume" so compared to each other they have equal probability of finding
% a point.
Ntheta = Ntheta./sum(Ntheta)*100;

% Create a theta and radius vector that we can plot
T1(2:3:(length(xtheta)-1)*3,1) = xtheta(1:end-1);
T1(3:3:(length(xtheta)-1)*3,1) = xtheta(2:end);
R1(2:3:(length(xtheta)-1)*3,1) = Ntheta(1:end-1);
R1(3:3:(length(xtheta)-1)*3,1) = Ntheta(1:end-1);

% Calculate the statistics for the phi angle
xphi = [-pi/2:pi/(options.ACPhiBins-1):pi/2]';
Nphi = histc(phi,xphi);

% The last bin is the count that exactly equals pi/2, so add that value to
% the bin before the last one.
Nphi(length(Nphi)-1) = Nphi(length(Nphi)-1)+Nphi(length(Nphi));

% Create a normalizing factor. For phi we are dealing with cones of
% interest.  So, in this case the normalizing factor is the volume of the
% two cones formed by the bin edges. This cone is a shell so it is 
% approximately the area of the cone times the delta phi, or the shell 
% width. The area is porportional to the difference in height.
r = sin(pi/2-xphi);
h = 1-(cos(pi/2-xphi));
sf = 1./abs(diff(h));
Nphi(1:length(Nphi)-1) = Nphi(1:length(Nphi)-1).*sf;
Nphi = Nphi./sum(Nphi)*100;

% Create the vectors for the polar plot
T2(2:3:(length(xphi)-1)*3,1) = xphi(1:end-1);
T2(3:3:(length(xphi)-1)*3,1) = xphi(2:end);
R2(2:3:(length(xphi)-1)*3,1) = Nphi(1:end-1);
R2(3:3:(length(xphi)-1)*3,1) = Nphi(1:end-1);

% Determine the max radius to use in the plots
maxr1 = max(R1);
maxr2 = max(R2);

% Plot the bars as a filled polar plot
subplot(1,3,2)
polarf(T1,R1,[],maxr1)
title('Theta')
xlabel('Angle (degrees)')
ylabel('Normalized count/volume (%)')

% Plot the bars as a filled polar plot
subplot(1,3,3)
polarf(T2,R2,[],maxr2)
title('Phi')
xlabel('Angle (degrees)')
ylabel('Normalized count/volume (%)')


% --------------------------------------------------------------------
function PlotMenu_Callback(hObject, eventdata, handles)
% Use the callback function to enable or disable menu items under the plot
% menu
data = get(gcf,'Userdata');
results = data{1};

% Check to see if the autocorrelation has been performed
ACHandles = [handles.ACTriHistogramMenu,handles.AutocorrelationHistogram,...
    handles.ACExclusionZoneMenu];
if(results.ACCALC)
    set(ACHandles,'enable','on')
else
    set(ACHandles,'enable','off')
end

% Check to see if the Nearest Neighbor calculation has been performed
NNHandles = [handles.NNHistogramMenu,handles.DelaunaySegmentHistogramMenu,...
    handles.DelaunayVolHistogramMenu,handles.DelaunayAreaHistogramMenu,...
    handles.FalseColorNNDist,handles.FalseColorMaxNNDist,...
    handles.FalseColorAvgNNDistMenu];
if(results.NNCALC)
    set(NNHandles,'enable','on')
else
    set(NNHandles,'enable','off')
end

% Check to see if the Voronoi Domains calculation has been performed
VoronoiHandles = [handles.VoronoiVolumeHistogram,handles.VoronoiAreaHistogram,...
    handles.VoronoiDirVectorsMenu,handles.FalseColorVoronoiVolMenu,...
    handles.FalseColorVoronoiAreaMenu];
if(results.VORONOICALC)
    set(VoronoiHandles,'enable','on')
else
    set(VoronoiHandles,'enable','off')
end


% --------------------------------------------------------------------
function ACExclusionZoneMenu_Callback(hObject, eventdata, handles)
% This function will plot the Autocorrelation Exclusion Zone as a delaunay
% tesselation and then calculate the direction vector of the resultant
% shape.
data = get(gcf,'Userdata');
results = data{1};
options = data{2};
PlotData = data{3};

% Find the points that are 2*r_eff away from (0,0,0)
A = find(sqrt(results.ACx.^2 + results.ACy.^2 + results.ACz.^2)<=2*results.ACreff);
p = [0,0,0;results.ACx(A),results.ACy(A),results.ACz(A)];
try
    T = delaunayn(p);
catch
    SpatialAnalysisMessage(lasterr)
    errordlg('There was a problem determining the exclusion zone',...
        'Autocorrelation Exclusion Zone ERROR','modal')
    return
end

    
% Find the tetrahedrons with the index 1 (0,0,0)
[A,B] = find(T==1);
A = unique(T(A,:));
A = setdiff(A,1);

% Get the points that make up the nearest neighbors and then find the
% facet indices
X = p(A,:);

% If we have more than 3 nearest neighbors we can continue to calculate
% the area and direction vectors for each tetrahedron (pyramid)
if(length(A)>3)
    [K,Vol] = convhulln(X);
    [Area,n] = pyramidarea(X,K-1,[0,0,0]);
end

reff = (Vol*3/4/pi)^(1/3);

% Open a figure and draw the delaunay tesselation that represents the
% exclusion zone
SelectAxes(hObject, eventdata, handles, 'PlotAxes')
delete(get(gca,'Children'))
hold off
if(~isnan(K))
    for b = 1:length(K)
        h(b) = patch(X(K(b,:),1),X(K(b,:),2),X(K(b,:),3),'g');
    end
end

% Calculate the direction vector scaled by the area of each facet
Vect = sum(repmat(Area,1,3).*abs(n));
Vect = Vect./min(Vect)-1;

norm = sqrt(sum(Vect.^2));
if(norm>1)
    Vect = Vect./norm;
end

% Plot the direction vector as a line through the patch
VectLine = [3;-3]*results.ACreff*Vect;
h(b+1) = line(VectLine(:,1),VectLine(:,2),VectLine(:,3),'color',[0 0 0]);

% Write the direction vector as a vector in the plot
h(b+2)=text(-2.8*results.ACreff, -3.2*results.ACreff, 0*results.ACreff,...
    {['V_{aez} = <',num2str(Vect(1),'%4.2f,'),num2str(Vect(2),'%4.2f,'),...
    num2str(Vect(3),'%4.2f'),'>'];
    ['r_{aez} = ',num2str(reff,'%4.2f'),options.Units]},'EdgeColor',[0 0  0],...
    'BackgroundColor',[1 1 1],'EraseMode','none','VerticalAlignment','top');


% Save the PlotData
PlotData.handles = h;
PlotData.type = 'ACExclusionZone';
PlotData.style = '';

data{3} = PlotData;
set(gcf,'UserData',data)

% Set the x,y, and z labels
PrintAxisLabels(hObject, eventdata, handles)
title('Autocorrelation Exclusion Zone')
axis equal
axis([-3 3 -3 3 -3 3]*results.ACreff)
view(3);
box on

%-------------------------------------------------------------------------
function ButtonDwnDataTable(hObject, eventdata, handles)
% This function will display the help text box for the datatable.  

curpt = get(gcf,'CurrentPoint');
txtpos = get(handles.NNPointClassTXT,'position');

% Set the default of the text box to be invisible and set the default width
% and height
visible = 'on';
w = 6.2; h = 18;
txtstr = {' '};

% Perform the action based on which analysis is visible in the data table
switch(get(handles.DataTable2Title,'value'))
    case 1
        % General
        switch(get(hObject,'string'))
            case 'Area'
                txtstr = {'Optical area of the measured point'};
            case 'Volume'
                txtstr = {'Volume of the measured point'};
            case 'Eccentricity'
                txtstr = {'Eccentricity of the measured point'};
            case 'Circularity'
                txtstr = {'Circularity of the measured point'};
            case 'Sphericity'
                txtstr = {'Sphericity of the measured point'};
            otherwise
                visible = 'off';
        end
        
    case {2,3}
        % Near Neighbor Analysis/Delaunay Analysis
        switch(get(hObject,'string'))
            case 'Near Neighbors'
                txtstr = {'Number of near neighbors'};
            case 'Avg. Distance'
                txtstr = {'Average distance of the near neighbors'};
            case 'Min Distance'
                txtstr = {'Minimum near neighbor distance'};
            case 'Max Distance'
                txtstr = {'Maximum near neighbor distance'};
            case 'Point Classification'
                txtstr = {...
                    '1 - Normal Point';...
                    '2 - Double Infected Point';...
                    '3 - Infected Point';...
                    '4 - Point on the Convex Hull'};
            case 'Current Filter'
                txtstr = {'Current filter for the Delaunay analysis'}; 
            case '# of Tetrahedra'
                txtstr = {'Number of Delaunay tetrahedra in the sample space'};
            case 'Tetrahedron Number'
                txtstr = {'Current Delaunay tetrahedron being rendered'};
            case 'Tetrahedron Points'
                txtstr = {'Points making up the current tetrahedron'};
            case 'Tetrahedron Volume'
                txtstr = {'Volume of the current tetrahedron'};
            case 'Tetrahedron Surf Area'
                txtstr = {'Surace area of the current tetrahedron'};
            otherwise 
                visible = 'off';
                
        end       
    case 4
        % Voronoi Analysis
        switch(get(hObject,'string'))
            case 'Voronoi Vertices'
                txtstr = {'Number of vertices in the Voronoi domain'};
            case 'Voronoi Facets'
                txtstr = {'Number of facets in the Voronoi domain'};
            case 'Voronoi Surface Area'
                txtstr = {'Total surface area of the Voronoi domain'};
            case 'Voronoi Volume'
                txtstr = {'Total volume of the Voronoi domain'};
            case 'Direction Vector'
                txtstr = {'Direction vector of the Voronoi domain'};
            case 'Current Filter'
                txtstr = {'Current filter for the Voronoi analysis'};
            case 'Point Classification'
                txtstr = {...
                    '1 - Normal Point';...
                    '2 - Double Infected Point';...
                    '3 - Infected Point';...
                    '4 - Point on the Convex Hull';...
                    '5 - Other (see userguide)'};
            otherwise 
                visible = 'off';
        end       
    case 5
        % Autocorrelation
        visible = 'off';
end

A = length(txtstr);
maxlen = 0;
for a = 1:A
    lentxt = length(txtstr{a});
    if(lentxt>maxlen)
        maxlen = lentxt;
    end
end

% Determine the x,y,w,h of the text box
txtpos(3) = w*maxlen;
txtpos(4) = h*A;
txtpos(1) = curpt(1);
txtpos(2) = curpt(2)-txtpos(4);
set(handles.NNPointClassTXT,'position',txtpos,'visible',visible,'string',txtstr)

%-------------------------------------------------------------------------
function ButtonUpFcn(hObject, eventdata, handles)
set(handles.NNPointClassTXT,'visible','off')

%--------------------------------------------------------------------------
function SUCCESS = SetupSpatialAnalysis()
% This function will compile the mex files

SUCCESS = true;

% Set some preferences
setpref('SpatialAnalysis','User','user');
setpref('SpatialAnalysisTips','Display',1);

try
    % Create a waitbar to let the user know what's going on
    hWB = waitbar(0,'Compiling Mex Files...');
    
    % Set the interpreter of the title to none so that it does not misread
    % the file names as TeX strings
    set(get(get(hWB,'Children'),'title'),'interpreter','none')

    % Find the folder where spatial analysis is kept
    MainFcn = which('SpatialAnalysis.m');
    [WorkDir,MainFcn,ext,versn] = fileparts(MainFcn);

    % Change to the 'C' folder under the working directory
    cd([WorkDir,filesep,'C'])

    %Compile: ACCorrectionFactor3.c    
    file1 = 'ACCorrectionFactor3.c';
    waitbar(.2,hWB,['Compiling ',file1,'...']);
    mex('-outdir',WorkDir,file1)

    %Compile: dmin3d.c
    file2 = 'dmin3d.c';
    waitbar(.4,hWB,['Compiling ',file2,'...']);
    mex('-outdir',WorkDir,file2)

    %Compile: kfg_funs.c
    file3 = 'kfg_funs.c';
    file4 = 'sphefrac.c';
    file5 = 'sint.c';
    file6 = 'f3.c';
    file7 = 'g3.c';
    file8 = 'k3.c';
    file9 = 'sphevol.c';
    waitbar(.6,hWB,['Compiling ',file3,'...']);
    mex('-outdir',WorkDir,file3,file4,file5,file6,file7,file8,file9)

    %Compile: pyramidvol.c
    file10 = 'pyramidvol.c';
    file11 = 'triarea3.c';
    waitbar(.75,hWB,['Compiling ',file10,'...']);
    mex('-outdir',WorkDir,file10,file11)

    %Compile: pyramidarea.c
    file12 = 'pyramidarea.c';
    waitbar(.85,hWB,['Compiling ',file12,'...']);
    mex('-outdir',WorkDir,file12,file11)
    
    %Compile: nnanalysis3.c
    file13 = 'tetraarea.c';
    file14 = 'nnanalysis3.c';
    waitbar(.95,hWB,['Compiling ',file14,'...']);
    mex('-outdir',WorkDir,file14,file11,file13)
    
    % Change back to the working directory
    cd(WorkDir)
    
    % Close the waitbar
    waitbar(1,hWB,'DONE')
    close(hWB)
    
catch
    % If we had trouble with any part of the setup, then create an error
    % dialog box and return
    errordlg(lasterr,'Spatial Analysis Setup ERROR','modal')
    SUCCESS = false;
    return
end

% If we did not have an error, tell the user that the setup was successful
hmb = msgbox('Spatial Analysis Setup was successful!','Spatial Analysis Setup',...
    'modal');

%--------------------------------------------------------------------------
function DeleteHistogramAxes(hObject, eventdata, handles)
% This function is called if the Histogram axes are deleted

% First check to see if we are deleting the entire figure
if(strcmp(get(gcf,'BeingDeleted'),'off'))
    % If we are only deleting the histogram axes, then make a copy and put
    % it right back into the figure, turn the selected property off and
    % save this new axis handle into the handles structure 
    C = copyobj(gca,gcf);
    set(C,'Selected','off')
    handles.HistogramAxes = C;
    guidata(gcf,handles)
end


% --------------------------------------------------------------------
function ToolsMenu_Callback(hObject, eventdata, handles)
% Check to see if certain analyses have been done so that we can enable or
% disable certain menu items

data = get(gcf,'Userdata');
results = data{1};

% Check to see if the Delaunay tessellation calculation has been performed
DelaunayHandles = [handles.ViewAllPyramidsMenu];
if(results.NNCALC)
    set(DelaunayHandles,'enable','on')
else
    set(DelaunayHandles,'enable','off')
end

% --------------------------------------------------------------------
function ViewAllPyramidsMenu_Callback(hObject, eventdata, handles)
% This function will go through  and display each Delaunay tetrahedron that
% is not filtered

% First thing to do is call the Delaunay menu function
DelaunayMenu_Callback(hObject, eventdata, handles)

data = get(gcf,'UserData');
results = data{1};
PlotData = data{3};

% Get the indices of the Delaunay tetrahedrons that are not filtered
B = GetFilteredDelaunayIndices(hObject, eventdata, handles, results,'vol');

% Check to see if we have filtered out all the tetrahedra
if(isempty(B))
    SpatialAnalysisMessage('No Delaunay Tetrahedra were found for the current filter');
    return
end

% Make the STOP Button visible so the user can stop this stuff!
set(handles.STOPButton,'visible','on','UserData',0)

% Now, go through and plot ALL the tetrahedrons
for i = B'
    if(get(handles.STOPButton,'UserData'))
        break
    end
    % Increase the tetrahedron we are on by 1
    results.CurTetra = i;

    % Save the results
    data{1} = results;
    set(gcf,'UserData',data)

    % Update the figure based on the current point
    UpdateOnCurrentPoint(hObject, eventdata, handles, i-1)

    % Get the data again
    data = get(gcf,'UserData');
end

% Make the STOP Button invisible again
set(handles.STOPButton,'visible','off')

% --------------------------------------------------------------------
function DelaunayMovieMenu_Callback(hObject, eventdata, handles)
data = get(gcf,'UserData');
results = data{1};
options = data{2};

pos = get(gcf,'position');
% Open a new figure
figure
origpos = get(gcf,'position');
set(gcf,'position',pos);

% Plot the delaunay domains as a wiremesh
tetramesh(results.T,results.points,'facecolor','none')
set(gcf,'color',[1 1 1])
set(gca,'xcolor',[1 1 1],'ycolor',[1 1 1],'zcolor',[1 1 1])

% Determine the trajectory path's initial, and 2 final destinations
InitialDest = [options.MinX,options.MaxY,options.MinZ];
FinalDest1 = ([options.MinX,options.MinY,options.MinZ] + ...
    [options.MaxX,options.MaxY,options.MaxZ])/2;
FinalDest2 = [options.MaxX,options.MinY,options.MinZ];


% Find the points closest to the initial point, mid point and final point
[mindist,InitPoint] = min(sum((results.points -...
        repmat(InitialDest,results.NumPoints,1)).^2,2));

[mindist,MidPoint] = min(sum((results.points -...
        repmat(FinalDest1,results.NumPoints,1)).^2,2));

[mindist,LastPoint] = min(sum((results.points -...
        repmat(FinalDest2,results.NumPoints,1)).^2,2));


% Set the initial destination
FinalDest = FinalDest1;
oldpoint = [];
i = InitPoint;
for count = 1:10
    [minpoint,A] = min(sum((results.points(results.NNIndices{i},:)-...
        repmat(FinalDest,results.NumNN(i),1)).^2,2));
    oldpoint(count) = i;
    i = results.NNIndices{i}(A);    
    if(i==MidPoint)
        break
    end
end
Points = [oldpoint];    


% Set the final destination
FinalDest = FinalDest2;
oldpoint = [];
i = MidPoint;
for count = 1:10
    [minpoint,A] = min(sum((results.points(results.NNIndices{i},:)-...
        repmat(FinalDest,results.NumNN(i),1)).^2,2));
    oldpoint(count) = i;
    i = results.NNIndices{i}(A);    

    if(i==LastPoint)
        break
    end
end
    
Points = [Points,oldpoint,LastPoint];    
t = [1:numel(Points)]';

t1 = [1:.03:numel(Points)]';

path = interp1(t,results.points(Points,:),t1,'spline');
x = path(:,1);
y = path(:,2);
z = path(:,3);

hold on
plot3(x,y,z,'r','linewidth',2)

% Plot the voronoi diagram for the midpoint
C = results.C;
V = results.V;
indices = C{MidPoint};
X = V(indices,:);
K = results.K{MidPoint};

% Create a patch (facet) for each voronoi surface
for b = 1:length(K)
    h(b) = patch(X(K(b,:),1),X(K(b,:),2),X(K(b,:),3),'g');
end
hP = plot3(X(:,1),X(:,2),X(:,3),'.r');

alpha(h,options.Transparency)
set(h,'edgecolor','none')

pathdiff = diff(path);

% Remember the original camera position, target and viewing angle
set(gca,'CameraPositionMode','manual','CameraTargetMode','manual',...
    'CameraViewAngleMode','auto');

% Pause for a seconds
pause(1)

for i = 1:length(t1)-1
    % Set the camera's position and target
    %set(gca,'CameraPosition',[x(i) y(i) z(i)])
    %set(gca,'CameraTarget',[x(i) y(i) z(i)]+pathdiff(i,:)*10)
    
    if(i==1)
        drawnow
        V = [x(1) y(1) z(1)]-([x(1) y(1) z(1)]+pathdiff(1,:)*10);
        [th,phi,r] = cart2sph(V(1),V(2),V(3));
        th = 90+th*180/pi;
        phi = phi*180/pi;
        [AZ,EL] = view;

        % Calculate the difference needed to move the azimuthal and
        % elevation angles
        dAZ = th-AZ;
        dEL = phi-EL;
        
        % Check to see if we are moving more than 180 degrees in the
        % azimuthal direction, if so, go the other direction
        if(dAZ>180)
            dAZ = dAZ-360;
        end
        
        % First change the camera position
        N = 50;
        for j = 1:N
            view(AZ+dAZ/N*j,EL+dEL/N*j)     
            drawnow
        end

        % Now, slowly change the camera position and camera target while
        % leaving the viewing angle on auto so that it gets the entire
        % scene
        dpos = [x(i) y(i) z(i)]-get(gca,'CameraPosition');
        dtarget = ([x(i) y(i) z(i)]+pathdiff(i,:)*10)-get(gca,'CameraTarget');
        pos = get(gca,'CameraPosition');
        target = get(gca,'CameraTarget');
        
        % Determine the width of the field of view.  This will be used
        % later to determine the camera viewing angle
        w = tand(get(gca,'CameraViewAngle'))*sqrt(sum((target-pos).^2));
        
        % Change the position and target
        N = 50;
        maxlog = 10;
        steps = log(1:(maxlog-1)/(N-1):maxlog)/log(maxlog)*(N-1)+1;
        for j = steps

            % Determine what the new camera viewing angle should be
            ang = atand(w/sqrt(sum(((target+dtarget/N*j) - (pos+dpos/N*j)).^2)));

            set(gca,'CameraPosition',pos+dpos/N*j,...
                'CameraTarget',target+dtarget/N*j,...
                'CameraViewAngle',ang)                        
            drawnow
        end
            
        % Now change the camera viewing angle 
        N = 10;
        initialview = get(gca,'CameraViewAngle');
        finalview = 100;
        dview = finalview-initialview;     
        
    else
        % If we are not on the first iteration, then we just need to change
        % the camera position and target while leaving the angle fixed
         set(gca,'CameraPosition',[x(i) y(i) z(i)])
         set(gca,'CameraTarget',[x(i) y(i) z(i)]+pathdiff(i,:)*10)
    end
    drawnow
end


% Go back to the original viewing position
%dpos = get(gca,'CameraPosition')-origpos;
dtarget = get(gca,'CameraTarget')-FinalDest1;
dangle = get(gca,'CameraViewAngle')-initialview;
N = 100;
for i = 1:N
    set(gca,'CameraTarget',get(gca,'CameraTarget')-dtarget/(N-1))
    drawnow
end

%--------------------------------------------------------------------------
function CONTINUE = CheckNNFilter()
% This function will check the current point versus the NN Filter to see if
% it is a filtered point or not
data = get(gcf,'UserData');
results = data{1};
options = data{2};

% Get the current point's index
a = results.CurPoint;

CONTINUE = 0;
switch(results.NNClass(a))
    case 1
        % Normal Point
        CONTINUE = 1;
    case 2
        % Double infected Point
        % Do NOT continue if the filter is 'Double Infected Points'
        if(~strcmp(options.NNFilter,'Double Infected Points'))
            CONTINUE = 1;
        end
    case 3
        % Infected point
        % We will continue only if the filter is set to none or convex hull
        if(strcmp(options.NNFilter,'Convex Hull') | ...
                strcmp(options.NNFilter,'None'))
            CONTINUE = 1;
        end
    case 4
        % Point on the convex hull
        % We will continue only if the filter is set to none
        if(strcmp(options.NNFilter,'None'))
            CONTINUE = 1;
        end 
end

%--------------------------------------------------------------------------
function CONTINUE = CheckDelaunayFilter()
% This function will check the current TETRAHEDRON point versus the 
% NN Filter to see if the tetrahedron has any filtered points or not
data = get(gcf,'UserData');
results = data{1};
options = data{2};

% Get the current tetrahreon point's index
a = results.CurTetra;

CONTINUE = 0;

switch(options.NNFilter)
    case 'None'
        % Render everything
        CONTINUE = 1;
    case 'Convex Hull'
        % Do not render anything with a Class 4 point
        if(~any(results.NNClass(results.T(a,:))==4))
            CONTINUE = 1;
        end
    case 'Infected Points'
        % Do not render anything with a Class 3 or 4 point
        if(~any(results.NNClass(results.T(a,:))>=3))
            CONTINUE = 1;
        end 
    case 'Double Infected Points'
        % Only render those with ALL Class 1 points
        if(all(results.NNClass(results.T(a,:))==1))
            CONTINUE = 1;
        end
end


% --------------------------------------------------------------------
function HistAutoScaleMenu_Callback(hObject, eventdata, handles)
% hObject    handle to HistAutoScaleMenu (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
function AutoScaleHistAxes_Callback(hObject, eventdata, handles)
% hObject    handle to AutoScaleHistAxes (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
data = get(gcf,'UserData');
results = data{1};
options = data{2};
PlotData = data{3};

% Remember the old options
oldoptions = options;

switch(PlotData.style)
    case 'NearestNeighbor'
        % Get the proper NN indices based on the filter
        A = GetFilteredNNIndices(hObject, eventdata, handles, results);

        % Calculate what the bin size should be based on the number of bins
        % selected, and the maximum NN distance
        options.NNBinSize = max(results.MinNNDist(A))./...
            options.NumNNBins;
        data{2} = options;
        set(gcf,'UserData',data)
        
        % Call the Nearest Neighbor Histogram
        NNHistogramMenu_Callback(hObject, eventdata, handles)

    case 'DelaunaySegLen'
        % Get the proper Delaunay indices based on the filter
        A = GetFilteredDelaunayIndices(hObject, eventdata, handles, results, 'Seg');
        
        % Calculate what the bin size should be based on the number of bins
        % selected, and the maximum seg length distance
        options.DelaunaySegLenBinSize = max(results.DelaunaySegLen(A))./...
            options.NumDelaunaySegLenBins;
        data{2} = options;
        set(gcf,'UserData',data)
        
        % Call the Delauany Segment Length Histogram
        DelaunaySegmentHistogramMenu_Callback(hObject, eventdata, handles)

    case 'DelaunayArea'
        % Get the proper Delaunay indices based on the filter
        A = GetFilteredDelaunayIndices(hObject, eventdata, handles, results, 'area');
        
        % Calculate what the bin size should be based on the number of bins
        % selected, and the maximum facet area
        options.DelaunayAreaBinSize = max(results.DelaunayArea(A))./...
            options.NumDelaunayAreaBins;
        data{2} = options;
        set(gcf,'UserData',data)
        
        % Call the Delauany Facet Area Histogram
        DelaunayAreaHistogramMenu_Callback(hObject, eventdata, handles)

    case 'DelaunayVolume'
         % Get the proper Delaunay indices based on the filter
        A = GetFilteredDelaunayIndices(hObject, eventdata, handles, results, 'vol');
        
        % Calculate what the bin size should be based on the number of bins
        % selected, and the maximum facet area
        options.DelaunayVolBinSize = max(results.DelaunayVolume(A))./...
            options.NumDelaunayVolBins;
        data{2} = options;
        set(gcf,'UserData',data)
        
        % Call the Delauany Facet Area Histogram
        DelaunayVolHistogramMenu_Callback(hObject, eventdata, handles)       
    
    case 'VoronoiSurfaceArea'
        % Get the proper Voronoi indices based on the filter
        A = GetFilteredVoronoiIndices(hObject, eventdata, handles, results);

        % Calculate what the bin size should be based on the number of bins
        % selected, and the maximum Voronoi Area
        options.VoronoiAreaBinSize = max(results.VoronoiArea(A))./...
            options.NumVoronoiAreaBins;
        data{2} = options;
        set(gcf,'UserData',data)
        
        % Call the Voronoi Area Histogram function
        VoronoiAreaHistogram_Callback(hObject, eventdata, handles)

    case 'VoronoiVolume'
        A = GetFilteredVoronoiIndices(hObject, eventdata, handles, results);

        % Calculate what the bin size should be based on the number of bins
        % selected, and the maximum Voronoi Volume
        options.VoronoiVolBinSize = max(results.VoronoiVolume(A))./...
            options.NumVoronoiVolBins;
        data{2} = options;
        set(gcf,'UserData',data)
        VoronoiVolumeHistogram_Callback(hObject, eventdata, handles)
end

% Save the old options back into the UserData
data{2} = oldoptions;
set(gcf,'UserData',data)

% -------------------------------------------------------------------------
function STOPButton_Callback(hObject, eventdata, handles)
% Just change the value of the STOP Button
set(hObject,'UserData',1)


