function varargout = TipOfTheDay(varargin)
% TIPOFTHEDAY Tip of the day for Spatial Analysis 3D
%  TipOfTheDay Creates or raises the existing figure window that shows the
%  tip of the day dialog box for Spatial Analysis 3D.  The figure is 
%  created as modal, so it must be closed in order to continue.
%
% See also: SPATIALANALYSIS

%  Below is the help information generated by the guide editor in Matlab
%
%      TIPOFTHEDAY, by itself, creates a new TIPOFTHEDAY or raises the existing
%      singleton*.
%
%      H = TIPOFTHEDAY returns the handle to a new TIPOFTHEDAY or the handle to
%      the existing singleton*.
%
%      TIPOFTHEDAY('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in TIPOFTHEDAY.M with the given input arguments.
%
%      TIPOFTHEDAY('Property','Value',...) creates a new TIPOFTHEDAY or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before TipOfTheDay_OpeningFunction gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to TipOfTheDay_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help TipOfTheDay

% Last Modified by GUIDE v2.5 25-Apr-2007 07:11:04

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @TipOfTheDay_OpeningFcn, ...
                   'gui_OutputFcn',  @TipOfTheDay_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before TipOfTheDay is made visible.
function TipOfTheDay_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to TipOfTheDay (see VARARGIN)

% Choose default command line output for TipOfTheDay
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

% Create the tips
Tips = {...
    'Double click on any plot to bring up the property (plot) editor.';...
    ['Use the arrow keys to cycle through points in the left axis. ',...
    'Up arrow moves forward 1 point, down arrow backward 1 point, ',...
    'right arrow forward 10 points and left arrow backward 10 points.'];...
    'Press the ''3'' key when plots are in a 2D view to take them back to a 3D view';...
    ['When sending a false color plot to another figure to use in a presentation, ',...
    'try restoring the markersize first under the Tools menu.'];...
    ['To view the axis in its true aspect ratio, select Equal Aspect Ratio, ',...
    'under the View menu inside the Tools menu.'];...
    ['Right click on histogram plots to select "Auto Scale Axes". ',...
    'This will automatically scale the bin size to the data of interest. ',...
    'The original bin size in the options menu will not be changed, however. ',...
    '(Does not work for Autocorrelation plots)'];...
    ['When in zoom or rotate mode, double click inside the axis to reset ',...
    'the plot back to its original view.'];...
    ['If a legend is on top of the plot, try moving it by double clicking ',...
    'on the plot to bring up the property editor, or holding down the left ',...
    'mouse button while over the legend.'];...
    ['In the tri-histogram plot, the phi plot goes from -pi to pi while ',...
    'the theta plot goes from 0 to 2pi.  To get the same angular division, ',...
    'the number of bins in the phi plot should be half of the theta plot.'];...
    ['The autocorrelation exclusion zone is the sum of all the Delaunay ',...
    'tetrahedrons formed by the near neighbors to the point (0,0,0)'];...
    ['The autocorrelation exclusion zone can be elongated in certain ',...
    'directions, this is represented by the vector, Vex.  See the ',...
    'userguide for more details.'];...
    ['The radius of the sphere with the same volume as the ',...
    'autocorrelation exclusion zone is given by rex'];...
    ['If you can not remember what the Point Classification stands ',...
    'for in the data table, try holding the mouse button down while '...
    'over the table entry and a pop-up window will appear to remind you. '...
    'This is true for many of the table entries.'];...
    ['When viewing all Delaunay tetrahedra, you can stop the entertainment ',...
    'by clicking on the STOP button or by pressing the ESC key.'];...
    ['Direction vector histograms with two idential bars on top of each other ',...
    'indicate that the direction vectors lie in the plane of those two directions.']};

% Create a preference for the tip number that we are currently showing
if(~ispref('SpatialAnalysisTips','TipNo'))
    setpref('SpatialAnalysisTips','TipNo',1)
end

% Create a preference as to whether or not to display the tip dialog
if(ispref('SpatialAnalysisTips','Display'))
    set(handles.ShowTipsCB,'value',getpref('SpatialAnalysisTips','Display'))
end

% Store the tips in the application data of the figure
setappdata(hObject,'Tips',Tips);

% Load the tip of the day file, and place it in the axis.  Then fix the
% axis ticks and colors, make sure the Tip figure window is current
% I changed the software to have the Tip and colormap embedded into the
% userdata of the figure
data = get(hObject,'UserData');
tip = data{1};
map = data{2};
figure(hObject)
image(tip)
colormap(map)
set(handles.axes2,'xcolor',[1 1 1],'ycolor',[1 1 1],'xtick',[],'ytick',[])

% Center the figure window
screensize = get(0,'MonitorPositions');
pos = get(hObject,'position');
pos(1) = max((screensize(3)-pos(3))/2,0);
pos(2) = max((screensize(4)-pos(4))/2,0);
set(hObject,'position',pos);

% Update the tip
UpdateTip(hObject, eventdata, handles) 


% --- Outputs from this function are returned to the command line.
function varargout = TipOfTheDay_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;

% -------------------------------------------------------------------------
function UpdateTip(hObject, eventdata, handles) 
% Get the tips from the application data
tips = getappdata(gcf,'Tips');

% Figure out what is the current tip
curtip = getpref('SpatialAnalysisTips','TipNo');

% Put the current tip into the TipTXT 
set(handles.TipTXT,'string',tips{curtip})

% -------------------------------------------------------------------------
function NextTipButton_Callback(hObject, eventdata, handles)
% Change the tip to display to be one higher then display the tip
tips = getappdata(gcbf,'Tips');

% Get the current tip number and then increase it by one.  Make sure the
% tip number is never greater than the number of tips we have and not equal
% to zero
curtip = getpref('SpatialAnalysisTips','TipNo');
setpref('SpatialAnalysisTips','TipNo',mod(curtip,length(tips))+1)

% Update the tip
UpdateTip(hObject, eventdata, handles) 

% -------------------------------------------------------------------------
function CloseButton_Callback(hObject, eventdata, handles)
% Close the figure window
close(gcbf)


% -------------------------------------------------------------------------
function ShowTipsCB_Callback(hObject, eventdata, handles)
% Will change the preference to show the tip of the day figure
setpref('SpatialAnalysisTips','Display',get(hObject,'value'))

