function varargout = saveselectedparams(varargin)
%SAVESELECTEDPARAMS  Save selected parameters to a tab delimited text file
%  SAVESELECTEDPARAMS(NAMES,DATA)  Calls the Save Selected Parameters
%  dialog.  The input variables are two cell arrays which represent the
%  data to be saved.  NAMES is a cell array of strings that describes the
%  DATA cell array.  The length of NAMES and DATA should be equal, and DATA
%  should consist of only double arrays.
%
%  H = SAVESELECTEDPARAMS(NAMES,DATA)  Returns the handle to the figure
%  window that is created.
%
%   See also: SAVEPARAMS, WAITFOR

% SAVESELECTEDPARAMS M-file for saveselectedparams.fig
%      SAVESELECTEDPARAMS, by itself, creates a new SAVESELECTEDPARAMS or raises the existing
%      singleton*.
%
%      H = SAVESELECTEDPARAMS returns the handle to a new SAVESELECTEDPARAMS or the handle to
%      the existing singleton*.
%
%      SAVESELECTEDPARAMS('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in SAVESELECTEDPARAMS.M with the given input arguments.
%
%      SAVESELECTEDPARAMS('Property','Value',...) creates a new SAVESELECTEDPARAMS or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before saveselectedparams_OpeningFunction gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to saveselectedparams_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help saveselectedparams

% Last Modified by GUIDE v2.5 09-Mar-2006 18:43:25

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @saveselectedparams_OpeningFcn, ...
                   'gui_OutputFcn',  @saveselectedparams_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before saveselectedparams is made visible.
function saveselectedparams_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to saveselectedparams (see VARARGIN)

% Choose default command line output for saveselectedparams
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes saveselectedparams wait for user response (see UIRESUME)
% uiwait(handles.figure1);

% Get the parameter names and associated data 
if(length(varargin)<2)
    error('Incorrect number of input variables')
    return
end

names = varargin{1};
data = varargin{2};

% Check to see if the user passed the variables correctly
if(length(names)~=length(data))
    error('Length of NAMES must equal length of DATA')
    return
end

% First sort the parameters into alphabetical order
[names,A] = sort(names);
data = data(A);

% Save the data within the figure
data2 = {names,data,[],[],names,data};
set(hObject,'UserData',data2,'Tag','SaveSelectedParamsFig')
set(handles.AllParametersLB,'value',1);

% Center the figure window
screensize = get(0,'MonitorPositions');
pos = get(hObject,'position');
pos(1) = max((screensize(3)-pos(3))/2,0);
pos(2) = max((screensize(4)-pos(4))/2,0);
set(hObject,'position',pos);

% Load the data
UpdateAllParams(hObject, eventdata, handles)

%--------------------------------------------------------------------------
function UpdateAllParams(hObject, eventdata, handles)
% This function will load the data into the figure window
data = get(gcf,'UserData');
names = data{1};
   
set(handles.AllParametersLB,'string',names)

%--------------------------------------------------------------------------
function UpdateSelectedParams(hObject, eventdata, handles)
% This function will load the data into the figure window
data = get(gcf,'UserData');
names = data{3};

set(handles.SelectedParametersLB,'string',names)

% --- Outputs from this function are returned to the command line.
function varargout = saveselectedparams_OutputFcn(hObject, eventdata, handles) 
% Get default command line output from handles structure
varargout{1} = handles.output;


%--------------------------------------------------------------------------
function AllParametersLB_CreateFcn(hObject, eventdata, handles)
% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


%--------------------------------------------------------------------------
function SelectedParametersLB_CreateFcn(hObject, eventdata, handles)
% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


%--------------------------------------------------------------------------
function SelectButton_Callback(hObject, eventdata, handles)
% This function will put a selected parameter down to the selected
% parameters listbox
data = get(gcf,'UserData');
namesAP = data{1};
dataAP = data{2};
namesSP = data{3};
dataSP = data{4};

% Get the number of the selected item
num = get(handles.AllParametersLB,'value');
len = length(namesAP);

% Check to see if the list is empty
if(len==0)
    return
end

% Put the selected item into the Selected Parameters list
namesSP = [namesSP,namesAP(num)];
dataSP = [dataSP,dataAP(num)];

% Take the item out of the All Parameters list
namesAP = namesAP([1:num-1,num+1:len]);
dataAP = dataAP([1:num-1,num+1:len]);

data{1} = namesAP;
data{2} = dataAP;
data{3} = namesSP;
data{4} = dataSP;

% Before we update, we must make sure the value in the List box is
% valid
if(num==len)
    set(handles.AllParametersLB,'value',max(num-1,1))
    set(handles.SelectedParametersLB,'value',1)
end

set(gcf,'userdata',data)

UpdateAllParams(hObject, eventdata, handles)
UpdateSelectedParams(hObject, eventdata, handles)

%--------------------------------------------------------------------------
function UnselectButton_Callback(hObject, eventdata, handles)
% This function will put a selected parameter back up into the unselected
% parameters listbox
data = get(gcf,'UserData');
namesAP = data{1};
dataAP = data{2};
namesSP = data{3};
dataSP = data{4};

% Get the number of the selected item
num = get(handles.SelectedParametersLB,'value');
len = length(namesSP);

% Check to see if the list is empty
if(len==0)
    return
end

% Put the selected item into the ALL Parameters list
namesAP = [namesAP,namesSP(num)];
dataAP = [dataAP,dataSP(num)];

% Take the item out of the Selected Parameters list
namesSP = namesSP([1:num-1,num+1:len]);
dataSP = dataSP([1:num-1,num+1:len]);

% Sort the ALL Parameters list
[namesAP,A] = sort(namesAP);
dataAP = dataAP(A);

data{1} = namesAP;
data{2} = dataAP;
data{3} = namesSP;
data{4} = dataSP;

% Before we update, we must make sure the value in the List box is
% valid
if(num==len)
    set(handles.SelectedParametersLB,'value',max(num-1,1))
    set(handles.AllParametersLB,'value',1)
end

set(gcf,'userdata',data)

% Update the listboxes
UpdateAllParams(hObject, eventdata, handles)
UpdateSelectedParams(hObject, eventdata, handles)


%--------------------------------------------------------------------------
function SaveButton_Callback(hObject, eventdata, handles)
% This function will save the selected parameters
data = get(gcf,'UserData');
namesSP = data{3};
dataSP = data{4};

% Check to see if there is a preference of where to save the data
% Check to see if the export data folder preference has been set
if(~ispref('ExportSettings','Folder'))
    addpref('ExportSettings','Folder',pwd)
end

% Remember the current working directory
curdir = pwd;

% Change the the last Export Directory
ExportFolder = getpref('ExportSettings','Folder');
cd(ExportFolder)

[filename,pathname] = uiputfile('*.txt','Save Selected Parameters...');

% If we returned a valid filename, then save the file
if(filename)
    file = [pathname,filename];
    fid = fopen(file,'wt');
    saveparams(fid,namesSP,dataSP)
    
    % Store the new pathname
    setpref('ExportSettings','Folder',pathname)
end
close(gcf)

% Change back to the old directory
cd(curdir)

%--------------------------------------------------------------------------
function CancelButton_Callback(hObject, eventdata, handles)
% This function will just close the window
close(gcf)


%--------------------------------------------------------------------------
function ResetButton_Callback(hObject, eventdata, handles)
% This function will put all the parameters back up to the unselected list
data = get(gcf,'UserData');
namesAP = data{1};
dataAP = data{2};
namesSP = data{3};
dataSP = data{4};

% Put the selected item into the ALL Parameters list
namesAP = [namesAP,namesSP];
dataAP = [dataAP,dataSP];

% Take the item out of the Selected Parameters list
namesSP = [];
dataSP = [];

% Sort the ALL Parameters list
[namesAP,A] = sort(namesAP);
dataAP = dataAP(A);

data{1} = namesAP;
data{2} = dataAP;
data{3} = namesSP;
data{4} = dataSP;

set(handles.SelectedParametersLB,'value',1)
set(handles.AllParametersLB,'value',1)

set(gcf,'userdata',data)

% Update the listboxes
UpdateAllParams(hObject, eventdata, handles)
UpdateSelectedParams(hObject, eventdata, handles)

%--------------------------------------------------------------------------
function SelectAllButton_Callback(hObject, eventdata, handles)
% This function will move ALL the data down to the selected box
data = get(gcf,'UserData');
namesAP = data{1};
dataAP = data{2};
namesSP = data{3};
dataSP = data{4};

% Put All the parameters into the selected list
namesSP = [namesSP,namesAP];
dataSP = [dataSP,dataAP];

% Take the items out of the All Parameters list
namesAP = [];
dataAP = [];

% Sort the Selected Parameters list
[namesSP,A] = sort(namesSP);
dataSP = dataSP(A);

data{1} = namesAP;
data{2} = dataAP;
data{3} = namesSP;
data{4} = dataSP;

% Set the listbox values to 1 so that we don't get an error
set(handles.SelectedParametersLB,'value',1)
set(handles.AllParametersLB,'value',1)

set(gcf,'userdata',data)

% Update the listboxes
UpdateAllParams(hObject, eventdata, handles)
UpdateSelectedParams(hObject, eventdata, handles)



